using System;
using System.Collections.Generic;
using System.Text;
using Habanero.BO;
using Habanero.Faces.Base;
using Replace_it.BO;
using MessageBoxButtons=Habanero.Faces.Base.MessageBoxButtons;
using MessageBoxIcon = Habanero.Faces.Base.MessageBoxIcon;

namespace Replace_it.UI
{
    public class OrdersGridManager<T> : ControlManager
        where T : PartsOrder, new()
    {
        private IReadOnlyGridControl _ordersGrid;
        private IReadOnlyGridControl _orderItemsGrid;

        public OrdersGridManager(IControlFactory controlFactory) : base(controlFactory)
        {
        }

        protected override void InitialiseControl()
        {
            BusinessObjectCollection<T> boCollection = new BusinessObjectCollection<T>();
            boCollection.LoadAll();
            _ordersGrid = _controlFactory.CreateReadOnlyGridControl();
            _ordersGrid.FilterControl.Visible = false;
            _ordersGrid.SetBusinessObjectCollection(boCollection);

            _orderItemsGrid = _controlFactory.CreateReadOnlyGridControl();
            _orderItemsGrid.Height = 200;
            _orderItemsGrid.FilterControl.Visible = false;
            SetOrderItemsCollection();

            _ordersGrid.Grid.BusinessObjectSelected += delegate { SetOrderItemsCollection(); };

            if (typeof(T) == typeof(CustomerOrder))
            {
                _ordersGrid.Buttons.AddButton("Dispatch", new EventHandler(DispatchClickedHandler));
            }
            else if (typeof(T) == typeof(SupplierOrder))
            {
                _ordersGrid.Buttons.AddButton("Receive", new EventHandler(ReceiveClickedHandler));
            }

            ICheckBox checkBox = _controlFactory.CreateCheckBox();
            checkBox.Text = "Undelivered orders only";
            checkBox.CheckedChanged += delegate
            {
                BusinessObjectCollection<T> col = new BusinessObjectCollection<T>();
                if (checkBox.Checked) col.Load("Delivered = 'false'", "");
                else col.LoadAll();
                _ordersGrid.SetBusinessObjectCollection(col);
            };

            BorderLayoutManager manager = _controlFactory.CreateBorderLayoutManager(_control);
            manager.AddControl(_ordersGrid, BorderLayoutManager.Position.Centre);
            manager.AddControl(_orderItemsGrid, BorderLayoutManager.Position.South);
            manager.AddControl(checkBox, BorderLayoutManager.Position.North);
        }

        private void SetOrderItemsCollection()
        {
            PartsOrder selectedPartsOrder = (PartsOrder)_ordersGrid.SelectedBusinessObject;
            if (selectedPartsOrder == null) return;
            _orderItemsGrid.SetBusinessObjectCollection(selectedPartsOrder.OrderItems);
        }

        private void DispatchClickedHandler(object sender, EventArgs e)
        {
            PartsOrder selectedOrder = (PartsOrder)_ordersGrid.SelectedBusinessObject;
            if (selectedOrder == null)
            {
                _controlFactory.ShowMessageBox("Please select an order.");
                return;
            }
            if (selectedOrder.Delivered.Value || selectedOrder.OrderItems.Count == 0) return;

            BusinessObjectCollection<ComputerPart> stockAvailable = new BusinessObjectCollection<ComputerPart>();
            stockAvailable.LoadAll();

            foreach (OrderItem orderItem in selectedOrder.OrderItems)
            {
                ComputerPart matchingPart = stockAvailable.FindByGuid(orderItem.ComputerPartID.Value);
                if (matchingPart == null)
                {
                    _controlFactory.ShowMessageBox("Could not find some of the stock on order. Please double-check.");
                    return;
                }

                if (orderItem.Quantity > matchingPart.Stock)
                {
                    _controlFactory.ShowMessageBox("Insufficient stock of part: " + matchingPart.PartCode);
                    return;
                }
            }

            foreach (OrderItem orderItem in selectedOrder.OrderItems)
            {
                ComputerPart matchingPart = stockAvailable.FindByGuid(orderItem.ComputerPartID.Value);
                matchingPart.Stock -= orderItem.Quantity;
                matchingPart.Save();
            }
            _controlFactory.ShowMessageBox("Stock dispatched and stock levels amended");
            selectedOrder.Delivered = true;
            selectedOrder.Save();
        }

        private void ReceiveClickedHandler(object sender, EventArgs e)
        {
            PartsOrder selectedPartsOrder = (PartsOrder)_ordersGrid.SelectedBusinessObject;
            if (selectedPartsOrder == null)
            {
                _controlFactory.ShowMessageBox("Please select an order.");
                return;
            }
            if (selectedPartsOrder.Delivered.Value || selectedPartsOrder.OrderItems.Count == 0) return;

            BusinessObjectCollection<ComputerPart> stockAvailable = new BusinessObjectCollection<ComputerPart>();
            stockAvailable.LoadAll();

            foreach (OrderItem orderItem in selectedPartsOrder.OrderItems)
            {
                ComputerPart matchingPart = stockAvailable.FindByGuid(orderItem.ComputerPartID.Value);
                if (matchingPart == null)
                {
                    _controlFactory.ShowMessageBox("Could not find some of the stock on order. Please double-check.");
                    return;
                }
            }

            foreach (OrderItem orderItem in selectedPartsOrder.OrderItems)
            {
                ComputerPart matchingPart = stockAvailable.FindByGuid(orderItem.ComputerPartID.Value);
                matchingPart.Stock += orderItem.Quantity;
                matchingPart.Save();
            }
            _controlFactory.ShowMessageBox("Stock received and stock levels amended");
            selectedPartsOrder.Delivered = true;
            selectedPartsOrder.Save();
        }
    }
}
