using System;
using System.Runtime.Serialization;

namespace FireStarter.CodeGeneration.Parser
{
    /// <summary>
    /// Provides an exception to throw when the parser encounters a
    /// problem while parsing a template for Habanero Firestarter
    /// </summary>
    public class ParserException : Exception
    {
        /// <summary>
        /// Constructor to initialise the exception
        /// </summary>
        public ParserException()
        {
        }

        /// <summary>
        /// Constructor to initialise the exception with a specific message
        /// to display
        /// </summary>
        /// <param name="message">The error message</param>
        public ParserException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Constructor to initialise the exception with a specific message
        /// to display and the inner exception specified
        /// </summary>
        /// <param name="message">The error message</param>
        /// <param name="inner">The inner exception</param>
        public ParserException(string message, Exception inner)
            : base(message, inner)
        {
        }

        /// <summary>
        /// Constructor to initialise the exception with the serialisation info
        /// and streaming context provided
        /// </summary>
        /// <param name="info">The serialisation info</param>
        /// <param name="context">The streaming context</param>
        protected ParserException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}