using System.IO;
using FireStarter.Base;
using FireStarterModeller.BO;
using Habanero.Base;

namespace FireStarter.CodeGeneration
{
    public class UIVWGProjectGenerator : ProjectGenerator
    {
        public override Result Generate()
        {
            CheckRequiredPropertiesSet();
            if (SolutionGenerator.SolutionTypeName == null) throw new PropertyMissingException("SolutionTypeName");
            Directory.CreateDirectory(ProjectPath);

            TemplateLoader vwgProjectTemplateLoader = new TemplateLoader();

            string contents = vwgProjectTemplateLoader.GetAssemblyInfo(ProjectName);
            Utilities.CreateFile(ProjectPath, "Properties\\AssemblyInfo.cs", contents);

            string projectFileName = Path.Combine(ProjectPath, ProjectName + ".csproj");
            ProjectFileManager projectFileManager = new ProjectFileManager(projectFileName, OutputType.Library, SolutionGenerator.CodeTemplate);

            projectFileManager.AddReference(Path.Combine(SolutionGenerator.HabaneroReferencePath, "Habanero.Base.dll"));
            projectFileManager.AddReference(Path.Combine(SolutionGenerator.HabaneroReferencePath, "Habanero.BO.dll"));
            projectFileManager.AddReference(Path.Combine(SolutionGenerator.HabaneroReferencePath, "Habanero.Faces.Base.dll"));
            projectFileManager.AddReference(Path.Combine(SolutionGenerator.HabaneroReferencePath, "Habanero.Faces.VWG.dll"));
            projectFileManager.AddReference(Path.Combine(SolutionGenerator.HabaneroReferencePath, "log4net.dll"));
            projectFileManager.AddReference(Path.Combine(SolutionGenerator.HabaneroReferencePath, "Gizmox.WebGUI.Forms.dll"));
            projectFileManager.AddReference(Path.Combine(SolutionGenerator.HabaneroReferencePath, "Gizmox.WebGUI.Common.dll"));
            projectFileManager.AddProjectReference(ModelSolution.UIProjectName, SolutionGenerator.SolutionDirectory);
            projectFileManager.AddProjectReference(ModelSolution.BOProjectName, SolutionGenerator.SolutionDirectory);
            projectFileManager.AddProjectReference(ModelSolution.LogicProjectName, SolutionGenerator.SolutionDirectory);

            if (GenerateControlFactory()) AddDefaultControlFactory(projectFileManager);

            projectFileManager.SaveProjectFile();
            return new Result(true);
        }

        internal bool GenerateControlFactory()
        {
            return SolutionGenerator.SolutionTypeName == GenerationTypes.SOLUTION_TYPE_DUALWINVWG;
        }

        private void AddDefaultControlFactory(ProjectFileManager projectFileManager)
        {
            //GenerationUtilities.GenerateCodeFile(projectFileManager, "Templates/UI_VWG_ControlFactory.txt",
            //    ProjectPath, ApplicationName + "ControlFactoryVWG.cs", CreateProjectModelUIVWG());

            UIVWGProjectTemplateLoader uivwgProjectTemplateLoader = new UIVWGProjectTemplateLoader();
            string contents = uivwgProjectTemplateLoader.GetControlFactory(ProjectName, SolutionGenerator.ModelSolution);
            string brokerName = Utilities.CreateFile(ProjectPath, SolutionGenerator.ModelSolution.ApplicationName + "ControlFactoryVWG.cs", contents);
            projectFileManager.AddCodeFile(brokerName);
        }

        private ProjectModelUIVWG CreateProjectModelUIVWG()
        {
            return new ProjectModelUIVWG
            {
                ProjectName = SolutionGenerator.ModelSolution.ApplicationName,
                UIBaseProjectName = GenerationUtilities.GetProjectName(ModelSolution.UIProjectName),
                UIVWGProjectName = ProjectName
            };
        }

        private class ProjectModelUIVWG : IModelElement
        {
            public string ProjectName { get; set; }
            public string UIVWGProjectName { get; set; }
            public string UIBaseProjectName { get; set; }

            #region Implementation of IModelElement

            public IModelElement Parent
            {
                get { return null; }
                set { throw new System.NotImplementedException(); }
            }

            #endregion
        }
    }
}