// ------------------------------------------------------------------------------
// This partial class was auto-generated for use with the Habanero Architecture.
// ------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using FireStarter.Base;
using Firestarter.DB.Schema.BO;
using Habanero.Base;
using Habanero.Base.Exceptions;
using Habanero.BO;
using Habanero.BO.ClassDefinition;
using Habanero.Util;

namespace FireStarterModeller.BO
{
    public interface IDMProperty:IBusinessObject
    {
        String PropertyName { get; set; }
        bool IsCompulsory { get; }
        int? OrdinalPosition { get; }
        DMPropertyType PropertyType { get; set; }
        IDMClass Class { get; set; }
        bool HasLookupList { get; }
        string PropertyTypeString { get; }
        bool IsPartOfObjectIdentity { get; }
        DBColumn MappedDBColumn { get; }
        string MappedDBColumnName { get; }
        bool IsPartOfSingleUniqueConstraint { get; }
        String DisplayName { get; set; }

//        Boolean? Compulsory { get; }
    }
    [Serializable]
    public partial class DMProperty : IModelProperty, IPropDef, IOrderableObject, IDMProperty, IMergableBo
    {
        
        public string SortOrderPropertyName { get; set;}

        public DMProperty() : this(null)
        {
            HasSetter = true;
           
        }

        protected internal DMProperty(ClassDef def) : base(def)
        {
            SetupPropertyUpdatesHandlers();
            SetupPropertyLinks();
            SetPropAuthorisationRules();
        }

        protected DMProperty(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            SetupPropertyLinks();
            SetupPropertyUpdatesHandlers();
            SetPropAuthorisationRules();
        }
        protected DMProperty(ConstructForFakes constructForFakes)
            : base(constructForFakes)
        {
        }
        private void SetPropAuthorisationRules()
        {
            IBOProp generatedProp = this.Props["Generated"];
            SetPropAuthorisation((BOProp) this.Props["PropertyName"], generatedProp);
            SetPropAuthorisation((BOProp) this.Props["PropertyTypeID"], generatedProp);
        }

        private static void SetPropAuthorisation(BOProp prop, IBOProp generatedProp)
        {
            BoPropAuthorisation boPropAuthorisation = new BoPropAuthorisation(prop, generatedProp);
            prop.SetAuthorisationRules(boPropAuthorisation);
        }

        private void SetupPropertyUpdatesHandlers()
        {
            IBOProp propertyNameProp = this.Props["PropertyName"];
            propertyNameProp.Updated += PropertyNameUpdatedHandler;
            IBOProp propertyTypeIdProp = this.Props["PropertyTypeID"];
            propertyTypeIdProp.Updated += PropertyTypeIdUpdatedHandler;
            IBOProp classIdProp = this.Props["ClassID"];
            classIdProp.Updated += ClassIdProp_OnUpdated;
        }

        private void ClassIdProp_OnUpdated(object sender, BOPropEventArgs e)
        {
            if (this.ClassID != null && this.PropertyType == null)
            {
                this.PropertyTypeString = DMPropertyType.PROP_TYPE_STRING;
            }
        }

        private void PropertyTypeIdUpdatedHandler(object sender, BOPropEventArgs e)
        {
            UpdateDefaultPropRuleParameters();
        }

        protected override void LoadBusinessObjectRules(IList<IBusinessObjectRule> boRules)
        {
            base.LoadBusinessObjectRules(boRules);
            boRules.Add(new DMPropertyNameNoSpecialCharsRule());
            boRules.Add(new DMPropertyNameNotSameAsClassName());
            boRules.Add(new DMPropertyNameDoesNotStartWithNumberRule());
            boRules.Add(new DMPropertyPropertyTypeSameAsBOLookupListClassIDType());
            boRules.Add(new DMPropertyNameRule());
            boRules.Add(new DMPropertyAutoIncrementingMustBeOfTypeInt());

        }

        private void UpdateDefaultPropRuleParameters()
        {
            Guid? valueBeforeLastEdit = (Guid?) this.Props["PropertyTypeID"].ValueBeforeLastEdit;
            Guid? previousPropertyTypeId = null;
            if (valueBeforeLastEdit != null)
            {
                previousPropertyTypeId = valueBeforeLastEdit;
            }
            if (this.PropertyTypeID == previousPropertyTypeId) return;
            //if (this.DefaultPropRule == null) return;
            if (this.DefaultPropRule == null) DefaultPropRule = CreateNewPropRule();

            bool hasNumericPropRuleParameters = false;
            if (this.PropertyType != null)
            {
                hasNumericPropRuleParameters = HasNumericPropRuleParameters(this.PropertyTypeString);
            }

            DMPropertyType previousPropertyType = null;
            IDMClass dmClass = this.Class;
            if (dmClass != null && dmClass.Solution != null)
            {
                previousPropertyType = dmClass.Solution.GetDMPropertyType(previousPropertyTypeId);
            }
            if (previousPropertyType != null)
            {
                if (hasNumericPropRuleParameters
                    && HasNumericPropRuleParameters(previousPropertyType.PropertyTypeFullName)) return;
            }
            ClearPropRuleParameters();
            DMPropRule defaultPropRule = this.DefaultPropRule;

            if (hasNumericPropRuleParameters)
            {
                CreatePropRuleParametersForNumberDateTime(defaultPropRule);
            }
            else if (this.PropertyType != null && this.PropertyTypeString == DMPropertyType.PROP_TYPE_STRING)
            {
                CreatePropRuleParametersForString(defaultPropRule);
            }
        }


        private static bool HasNumericPropRuleParameters(string propertyTypeId)
        {
            if (propertyTypeId == DMPropertyType.PROP_TYPE_INT32 || propertyTypeId == DMPropertyType.PROP_TYPE_DECIMAL
                || propertyTypeId == DMPropertyType.PROP_TYPE_SINGLE
                || propertyTypeId == DMPropertyType.PROP_TYPE_DOUBLE
                || propertyTypeId == DMPropertyType.PROP_TYPE_DATETIME)
            {
                return true;
            }
            return false;
        }

        private void ClearPropRuleParameters()
        {
            BusinessObjectCollection<DMPropRuleParameter> propRuleParameters = this.DefaultPropRule.PropRuleParameters;
            for (int i = propRuleParameters.Count - 1; i >= 0; i--)
            {
                DMPropRuleParameter propRuleParameter = propRuleParameters[i];
                if (propRuleParameter.Status.IsNew)
                {
                    propRuleParameter.CancelEdits();
                    propRuleParameters.Remove(propRuleParameter);
                }
                else
                {
                    propRuleParameter.MarkForDelete();
                }
            }
        }


        private void PropertyNameUpdatedHandler(object sender, BOPropEventArgs e)
        {
            DMPropRule defaultPropRule = this.DefaultPropRule;
            if (defaultPropRule == null) return;
            defaultPropRule.PropRuleName = e.Prop.PropertyValueString;
        }

        private void SetupPropertyLinks()
        {
            new PropertyLink<string, string>(this, "PropertyName", "MappedDBColumnName", PropNameToFieldNameTransform);
            new PropertyLink<string, string>(this, "PropertyName", "DisplayName", GetDisplayNameFromPropertyName);
        }

        public static string GetDisplayNameFromPropertyName(string value)
        {
            return StringUtilities.DelimitPascalCase(Convert.ToString(value), " ");
        }

        private DMPropRule CreateNewPropRule()
        {
            DMPropRule newPropRule = new DMPropRule {DMProperty = this, PropRuleName = this.PropertyName};
            CreatePropRuleParametersForString(newPropRule);
            return newPropRule;
        }

        private static void CreatePropRuleParametersForString(DMPropRule newPropRule)
        {
            CreateParameter(newPropRule, "minLength", 1);
            CreateParameter(newPropRule, "maxLength", 2);
            CreateParameter(newPropRule, "patternMatch", 3);
            CreateParameter(newPropRule, "patternMatchMessage", 4);
        }

        private static void CreatePropRuleParametersForNumberDateTime(DMPropRule newPropRule)
        {
            CreateParameter(newPropRule, "min", 1);
            CreateParameter(newPropRule, "max", 2);
        }

        private static void CreateParameter(DMPropRule newPropRule, string keyName, int position)
        {
            DMPropRuleParameter propRuleParameter = newPropRule.PropRuleParameters.CreateBusinessObject();
            propRuleParameter.KeyName = keyName;
            propRuleParameter.OrdinalPosition = position;
        }

        private string PropNameToFieldNameTransform(string propName)
        {
            return this.MappedDBColumnID == null ? GetColumnNameFromPropName(propName) : this.MappedDBColumnName;
        }

        public static string GetColumnNameFromPropName(string propName)
        {
            return propName;
        }
        public virtual IDMClass Class
        {
            get
            {
                return Relationships.GetRelatedObject<DMClass>("Class");
            }
            set
            {
                Relationships.SetRelatedObject("Class", value);
            }
        }

        /// <summary>
        /// returns true if this <see cref="DMProperty"/> is part of the <see cref="ObjectIdentity"/> its <see cref="DMClass"/>
        /// </summary>
        public bool IsPartOfObjectIdentity
        {
            get { return this.ObjectIdentity != null; }
        }

        public bool HasLookupList
        {
            get { return this.LookupList != null; }
        }

        public override string ToString()
        {
            if (String.IsNullOrEmpty(this.PropertyName))
            {
                return this.PropertyID == null ? "-unnamed DMProperty-" : Convert.ToString(this.PropertyID);
            }
            return this.PropertyName;
            // return this.Class == null ? this.PropertyName : this.Class + " - " + this.PropertyName;
        }

        public override bool IsDeletable(out string message)
        {
            base.IsDeletable(out message);
            if (this.Class != null && !string.IsNullOrEmpty(message)) message = "The DMProperty on DMClass '" + this.Class.ToString() + "' " + message;
            if (this.Generated.GetValueOrDefault())
            {
                message += "The property '" + this.ToString() + "' cannot be deleted as it has already been exported.";
                return false;
            }
            BusinessObjectCollection<DMUniqueConstraintProperty> uniqueConstraintProperties =  this.DMUniqueConstraintProperties;
            foreach (DMUniqueConstraintProperty uniqueConstraintProperty in uniqueConstraintProperties)
            {
                DMUniqueConstraint uniqueConstraint = uniqueConstraintProperty.UniqueConstraint;
                if (uniqueConstraint == null) continue;
                if (!uniqueConstraint.Status.IsDeleted)
                {
                    message = message + "The property '" + this.ToString()
                              + "' cannot be deleted as it is included in the Unique Constraint '"
                              + uniqueConstraint.ToString() + "'" + Environment.NewLine;
                    return false;
                }
            }
            BusinessObjectCollection<DMRelationshipProperty> ownerProperties = this.RelationshipProperties;
            foreach (DMRelationshipProperty relationshipProperty in ownerProperties)
            {
                IDMRelationship dmRelationship = relationshipProperty.Relationship;
                if (dmRelationship == null) continue;
                if (!dmRelationship.Status.IsDeleted)
                {
                    message = message + "The property '" + this.ToString()
                              + "' cannot be deleted as it forms part of the Relationship '" + dmRelationship.ToString()
                              + "'" + Environment.NewLine;
                    return false;
                }
            }


            BusinessObjectCollection<DMRelationshipProperty> relationshipProperties = this.RelationshipProperties;
            foreach (DMRelationshipProperty relationshipProperty in relationshipProperties)
            {
                IDMRelationship dmRelationship = relationshipProperty.Relationship;
                if (dmRelationship == null) continue;
                if (!dmRelationship.Status.IsDeleted)
                {
                    message = message + "The property '" + this.ToString()
                              + "' cannot be deleted as it forms part of the Relationship '" + dmRelationship.ToString()
                              + "'" + Environment.NewLine;
                    return false;
                }
            }
            return string.IsNullOrEmpty(message);
        }

        protected override void UpdateObjectBeforePersisting(ITransactionCommitter transactionCommitter)
        {
            DMPropRule defaultPropRule = this.DefaultPropRule;
            if (this.Status.IsDeleted || (defaultPropRule != null && defaultPropRule.PropRuleParameters.Count == 0))
            {
                if (defaultPropRule != null)
                {
                    defaultPropRule.MarkForDelete();
                    transactionCommitter.AddBusinessObject(defaultPropRule);
                    this.DefaultPropRule = null;
                }
            }
            base.UpdateObjectBeforePersisting(transactionCommitter);
        }

        public int Order
        {
            get { return this.OrdinalPosition == null ? 0 : this.OrdinalPosition.Value; }
        }

        public string PropertyTypeString
        {
            get
            {
                DMPropertyType propertyType = this.PropertyType;
                IDMClass dmClass = this.Class;
                if (propertyType == null && dmClass != null && dmClass.Solution != null)
                {
                    this.PropertyType = dmClass.Solution.GetDMPropertyType("System.String");
                }
                if (propertyType == null)
                {
                    throw new HabaneroDeveloperException
                        (String.Format("The property type for the property '{0}' has not been set.", this.PropertyName),
                         "");
                }
                return PropertyType.PropertyTypeFullName;
            }
            set
            {
                IDMClass dmClass = this.Class;
                if (dmClass == null) return;
                IDMSolution solution = dmClass.Solution;
                if (solution == null) return;
                DMPropertyType type = solution.GetDMPropertyType(value);
                this.PropertyType = type;
                if (type != null && type.PropertyTypeFullName == "System.Guid")
                {
                    //                    BusinessObjectCollection<DMPropRule> clone = this.PropRules.Clone();
                    //                    for (int i = clone.Count-1; i >= 0; i--)
                    //                    {
                    //                        DMPropRule propRule = clone[i];
                    //                        propRule.MarkForDelete();
                    //                        propRule.Save();
                    //                    }
                    DMPropRule defaultPropRule = this.DefaultPropRule;
                    if (defaultPropRule != null)
                    {
                        defaultPropRule.MarkForDelete();
                        defaultPropRule.Save();
                    }

                    this.DefaultPropRule = null;
                }
            }
        }


        private string GetPropRuleValue(string propRuleKey)
        {
            DMPropRule defaultPropRule = this.DefaultPropRule;
            if (defaultPropRule != null)
            {
                BusinessObjectCollection<DMPropRuleParameter> propRuleParameters = defaultPropRule.PropRuleParameters;
                foreach (DMPropRuleParameter propRuleParameter in propRuleParameters)
                {
                    if (propRuleParameter.KeyName == propRuleKey)
                    {
                        return propRuleParameter.Value ?? "";
                    }
                }
            }

            //            foreach (DMPropRule propRule in this.PropRules)
            //            {
            //                //if (this.DefaultPropRule == null) return "";
            //                //BusinessObjectCollection<DMPropRuleParameter> propRuleParameters = this.DefaultPropRule.PropRuleParameters;
            //                BusinessObjectCollection<DMPropRuleParameter> propRuleParameters = propRule.PropRuleParameters;
            //                foreach (DMPropRuleParameter propRuleParameter in propRuleParameters)
            //                {
            //                    if (propRuleParameter.KeyName == propRuleKey)
            //                    {
            //                        if (propRuleParameter.Value == null) return "";
            //                        return propRuleParameter.Value;
            //                    }
            //                }
            //            }
            return "";
        }

        public bool HasSetter { get; set; }

        public bool IsCompulsory
        {
            get { return this.Compulsory.GetValueOrDefault(); }
        }

        //public string SortOrderPropertyName
        //{
        //    get
        //    {
        //        return this.Class.ClassNameBO + "." + this.PropertyName;
        //    }
            
        //}


        #region Implementation of IModelElement

        IModelElement IModelElement.Parent
        {
            get { return this.Class as DMClass; }
            set
            {
                throw new HabaneroDeveloperException
                    ("Set the parent of the Property by setting the related Class object", "");
            }
        }

        #endregion

        #region Implementation of IModelProperty

        string IModelProperty.PropertyTypeString
        {
            get { return PropertyTypeString; }
          
        }

        bool IModelProperty.HasSetter
        {
            get
            {
                return HasSetter;
            }
            set { HasSetter = value; }
        }

        bool IModelProperty.IsCompulsory
        {
            get { return IsCompulsory; }
        }

        string IModelProperty.PropertyName
        {
            get { return this.PropertyName; }
            set { this.PropertyName = value; }
        }

        string IModelProperty.DefaultValue
        {
            get { return this.DefaultValue; }
        }

        string IModelProperty.Max
        {
            get
            {
                string maxValue = GetPropRuleValue("maxLength");
                if (maxValue == "") maxValue = GetPropRuleValue("max");
                return maxValue;
            }
        }

        string IModelProperty.Min
        {
            get
            {
                string minValue = GetPropRuleValue("minLength");
                if (minValue == "") minValue = GetPropRuleValue("min");
                return minValue;
            }
        }

        public string LookupListTypeString
        {
            get { return Convert.ToString(((IModelProperty) this).LookupListType);}
        }

        ModelLookupType IModelProperty.LookupListType
        {
            get
            {
                DMLookupList dmLookupList = this.LookupList;
                if (dmLookupList == null) return ModelLookupType.None;
                if (dmLookupList.GetType() == typeof (DMLookupListBusinessObject))
                {
                    return ModelLookupType.BusinessObject;
                }
                if (dmLookupList.GetType() == typeof (DMLookupListSimple))
                {
                    return ModelLookupType.Simple;
                }
                if (dmLookupList.GetType() == typeof (DMLookupListDatabase))
                {
                    return ModelLookupType.Database;
                }
                return ModelLookupType.None;
            }
        }

//This is only implemented so as to support the interface IPropDef
        public bool IsValueValid(object propValue, ref string errorMessage)
        {
            return true;
        }

        string IModelProperty.DisplayName
        {
            get { return DisplayName; }
        }
        IModelPropertyType IModelProperty.PropertyType
        {
            get { return this.PropertyType; }
        }

        string IModelProperty.ReadWriteRule
        {
            get
            {
                if (!this.ReadWriteRule.HasValue)
                {
                    throw new HabaneroDeveloperException
                        (String.Format("ReadWriteRule has a null value for the property '{0}'", this.PropertyName), "");
                }
                return this.ReadWriteRule.ToString();
            }
        }

        string IModelProperty.DatabaseField
        {
            get { return this.MappedDBColumnName; }
        }

        string IModelProperty.Description
        {
            get { return this.Description; }
        }

        bool IModelProperty.AutoIncrementing
        {
            get { return this.AutoIncrementing ?? false; }
        }

        bool IModelProperty.HasLength
        {
            get { return this.Length == null ? false : this.Length.Value > 0 && this.Length.Value < Int32.MaxValue; }
        }

        int IModelProperty.Length
        {
            get { return this.Length ?? 0; }
        }

        bool IModelProperty.KeepValuePrivate
        {
            get { return this.KeepValuePrivate ?? false; }
        }

        public bool GenerateCode
        {
            get
            {
                return ((bool)(base.GetPropertyValue("GenerateCode")));
            }
            set
            {
                base.SetPropertyValue("GenerateCode", value);
            }
        }
        #endregion

        #region Implementation of IPropDef

        IClassDef IPropDef.ClassDef
        {
            get { return (IClassDef) this.Class; }
            set { this.Class = (IDMClass) value; }
        }

        string IPropDef.DisplayNameFull
        {
            get { return DisplayName; }
        }

        public virtual string ClassName
        {
            get { return this.Class == null? "": this.Class.ClassNameBO; }
        }

        bool IPropDef.KeepValuePrivate
        {
            get { return KeepValuePrivate ?? false; }
        }

        bool IPropDef.Persistable
        {
            get { return true; }
            set{}
        }

        string IPropDef.PropertyName
        {
            get { return PropertyName; }
            set { PropertyName = value; }
        }

        ILookupList IPropDef.LookupList
        {
            get { return this.LookupList;  }
            set { this.LookupList = (DMLookupList) value;}
        }

        PropReadWriteRule IPropDef.ReadWriteRule
        {
            get { return ReadWriteRule.GetValueOrDefault(); }
            set { ReadWriteRule = value; }
        }

        bool IPropDef.AutoIncrementing
        {
            get { return AutoIncrementing ?? false; }
            set { AutoIncrementing = value; }
        }

        int IPropDef.Length
        {
            get { return Length ?? 0; }
        }
        
        string IPropDef.PropertyTypeName
        {
            get { return this.PropertyType.PropertyTypeName; }
            set { throw new NotImplementedException(); }
        }

        Type IPropDef.PropertyType
        {
            get
            {
                Type propertyType = null;
                TypeLoader.LoadClassType(ref propertyType, this.PropertyType.AssemblyName, this.PropertyType.PropertyTypeName,
                   "PropertyType", "DMProperty");
                return propertyType;
            }
            set
            {
                throw new NotImplementedException(); 
//                this.PropertyType = value;
            }
        }

        List<IPropRule> IPropDef.PropRules
        {
            get {
                return new List<IPropRule> {this.DefaultPropRule};
            }
        }

        string IPropDef.DatabaseFieldName
        {
            get { return this.MappedDBColumnName; }
            set { this.MappedDBColumnName = value; }
        }

        object IPropDef.DefaultValue
        {
            get { return DefaultValue; }
            set { DefaultValue = Convert.ToString(value); }
        }

        string IPropDef.DefaultValueString
        {
            get { return DefaultValue; }
            set { DefaultValue = value; }
        }

        bool IPropDef.Compulsory
        {
            get { return Compulsory ?? false; }
            set { Compulsory = value; }
        }


        void IPropDef.AddPropRule(IPropRule rule)
        {
            this.DefaultPropRule = (DMPropRule) rule;
        }

        bool IPropDef.HasLookupList()
        {
            return HasLookupList;
        }

        IPropertyComparer<T> IPropDef.GetPropertyComparer<T>()
        {
            throw new NotImplementedException();
        }

        IBOProp IPropDef.CreateBOProp(bool assignDefaultValue)
        {
            throw new NotImplementedException();
        }

        bool IPropDef.TryParsePropValue(object valueToParse, out object returnValue)
        {
            throw new NotImplementedException();
        }

        string IPropDef.ConvertValueToString(object value)
        {
            throw new NotImplementedException();
        }

        IPropDef IPropDef.Clone()
        {
            throw new NotImplementedException();
        }

        string IPropDef.DisplayName
        {
            get { return DisplayName; }
        }

        string IPropDef.Description
        {
            get
            {
                return this.Description;
            }
            set
            {
                this.Description = value;
            }
        }

        string IPropDef.PropertyTypeAssemblyName
        {
            get { return this.PropertyType.AssemblyName; }
            set { throw new NotImplementedException(); }
        }

        #endregion


        public bool IsPartOfSingleRelationship()
        {
            DMClass dmClass = (DMClass) this.Class;
            if (dmClass == null) return false;
            foreach (DMRelationship relationship in dmClass.OwnerRelationships)
            {
                if (relationship.Cardinality == Cardinality.Multiple) continue;
                foreach (DMRelationshipProperty relationshipProperty in relationship.RelationshipProperties)
                {
                    if (relationshipProperty.OwnerProperty == this) return true;
                }
            }
            return false;
        }

        /// <summary>
        /// To check if this property is part of a UniqueConstraint that
        /// is determined entirely by this property i.e. any one of the UniqueConstraints
        /// that this property participates in is not Composite.
        /// </summary>
        public bool IsPartOfSingleUniqueConstraint
        {
            get
            {
                return this.DMUniqueConstraintProperties.Any(ucProp => ucProp.IsPartOfSingleUniqueConstraint);
            }
        }

        public bool IsPartOfRelationshipToClass(IDMClass dmClass)
        {
            return this.RelationshipProperties.Any(relProp => relProp.Relationship.RelatedClass == dmClass);
        }

        
        public virtual void RenameProperty()
        {
            this.MappedDBColumnName = GetColumnNameFromPropName(this.PropertyName);
            this.DisplayName = GetDisplayNameFromPropertyName(this.PropertyName);
        }

        #region Implementation of IMergableBo

        ChangeType IMergableBo.PendingChangeType
        {
            get;
            set;
        }

        bool IMergableBo.PendingChangeApproved
        {
            get { return true; }
            set { }
        }

        #endregion
    }

    /// <summary>
    /// This has been creates such that we have a collection that implements IPropDefCol.
    /// This is used by the DMDefClassFactory for the loading of the Project from ClassDef.xml.
    /// </summary>
    public class DMPropertyCollection : BusinessObjectCollection<DMProperty>, IPropDefCol
    {
        IBOPropCol IPropDefCol.CreateBOPropertyCol(bool isNewObject)
        {
            throw new System.NotImplementedException();
        }

        void IPropDefCol.Add(IPropDef propDef)
        {
            this.Add((DMProperty)propDef);
        }

        IPropDefCol IPropDefCol.Clone()
        {
            throw new System.NotImplementedException();
        }

        IPropDefCol IPropDefCol.Clone(bool clonePropDefs)
        {
            throw new System.NotImplementedException();
        }

        bool IPropDefCol.Contains(IPropDef propDef)
        {
            return ((IPropDefCol)this).Contains(propDef.PropertyName);
        }

        bool IPropDefCol.Contains(string propertyName)
        {
            return this.Find(property => property.PropertyName == propertyName) != null;
        }

        void IPropDefCol.Remove(IPropDef propDef)
        {
            throw new System.NotImplementedException();
        }

        public void Add(IPropDefCol propDefCol)
        {
            throw new System.NotImplementedException();
        }

        IPropDef IPropDefCol.this[string propertyName]
        {
            get { return this.Find(property => property.PropertyName == propertyName); }
        }


        IEnumerator<IPropDef> IEnumerable<IPropDef>.GetEnumerator()
        {
            List<IPropDef> col = new List<IPropDef>();
            for (int i = 0; i < base.Count; i++)
            {
                col.Add(base[i]);
            }
            return col.GetEnumerator();
        }


    }
}