// ------------------------------------------------------------------------------
// This class was auto-generated for use with the Habanero Enterprise Framework.
// ------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using FireStarter.Base;
using Habanero.Base;
using Habanero.Base.Exceptions;
using Habanero.BO;
using Habanero.BO.ClassDefinition;

namespace FireStarterModeller.BO
{

    public interface IDMSolution:IBusinessObject
    {
        BusinessObjectCollection<DMPropertyType> PropertyTypes { get; }
        BusinessObjectCollection<DMAssembly> Assemblies { get; }
        BusinessObjectCollection<DMClass> AllClasses { get; }
        DMAssembly DefaultAssembly { get; set; }
        DMPropertyType GetDMPropertyType(Guid? propertyTypeId);
        DMPropertyType GetDMPropertyType(string fullTypeName);
        DMPropertyType GetDMPropertyType(string assemblyName, string typeName);
        DMAssembly CreateAssembly(string assemblyName);
        IList<IBOError> Validate();
        UITypeDefinition GetUIFilterControlType(string filterTypeAssembly, string filterType);
    }
    [Serializable]
    public partial class DMSolution : IModelSolution, IDMSolution, IMergableBo
    {
        private BusinessObjectCollection<DMClass> _classes;

        protected DMSolution(ConstructForFakes constructForFakes)
            : base(constructForFakes)
        {
        }

        public string ProjectFileName { get; set; }

        // ReSharper disable DoNotCallOverridableMethodsInConstructor
        public DMSolution(string solutionName) : this()
        {
            this.SolutionName = solutionName;
        }
        // ReSharper restore DoNotCallOverridableMethodsInConstructor

        public DMSolution()
        {
            //this.SetTransactionLog(new TransactionLogTable());
            SetupSolution();
        }

        protected DMSolution(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            SetupSolution();
        }

        protected internal DMSolution(ClassDef def) : base(def)
        {
            SetupSolution();
        }

        private void SetupSolution()
        {
            BOProp prop = (BOProp) this.Props["SolutionName"];
            BoPropAuthorisation boPropAuthorisation = new BoPropAuthorisation(prop, this.Props["Generated"]);
            prop.SetAuthorisationRules(boPropAuthorisation);
        }

        public override string ToString()
        {
            if (string.IsNullOrEmpty(SolutionName))
            {
                return this.SolutionID == null ? "-unnamed solution-" : Convert.ToString(this.SolutionID);
            }
            return this.SolutionName;
        }


        public virtual BusinessObjectCollection<DMClass> AllClasses
        {
            get
            {
                BusinessObjectCollection<DMAssembly> assemblies = this.Assemblies;
                foreach (DMAssembly dmAssembly in assemblies)
                {
                    RegisterForClassesEvents(dmAssembly);
                }
                assemblies.BusinessObjectAdded += Assemblies_OnBusinessObjectAdded;
                if (_classes == null)
                {
                    _classes = new BusinessObjectCollection<DMClass>();
                    assemblies.ForEach(assembly => _classes.Add(assembly.Classes));
                }
                _classes.Sort("ClassNameBO", true, true);
                return _classes;
            }
        }

        private void RegisterForClassesEvents(DMAssembly dmAssembly)
        {
            dmAssembly.Classes.BusinessObjectAdded += (sender, e) => _classes.Add(e.BusinessObject);
            dmAssembly.Classes.BusinessObjectRemoved += (sender, e) => _classes.Remove(e.BusinessObject);
        }

        private void Assemblies_OnBusinessObjectAdded(object sender, BOEventArgs<DMAssembly> e)
        {
            RegisterForClassesEvents(e.BusinessObject);
        }

        public static DMSolution Find(string name)
        {
            return Broker.GetBusinessObject<DMSolution>(string.Format("SolutionName = '{0}'", name));
        }

        #region Implementation of IModelElement

        IModelElement IModelElement.Parent
        {
            get { return null; }
            set { throw new HabaneroDeveloperException("A solution cannot have a parent", ""); }
        }

        #endregion

        #region Implementation of IModelSolution

        /// <summary>
        /// Gets all the model classes available to the solution, amalgamated
        /// from all the assemblies
        /// </summary>
        IList<IModelClass> IModelSolution.ModelClasses
        {
            get
            {
                List<IModelClass> modelClasses = new List<IModelClass>();
                BusinessObjectCollection<DMClass> classes = this.AllClasses;
                foreach (DMClass dmClass in classes)
                {
                    modelClasses.Add(dmClass);
                }
                return modelClasses;
            }
        }

        public string UIVWGProjectName { get; set; }
        //TODO Eric 27 Aug 2009: need to be persisted like the others or all moved out entirely
        public string UIWinProjectName { get; set; }
        public string ApplicationName { get; set; }

        public IList<IModelAssembly> ModelAssemblies
        {
            get
            {
                List<IModelAssembly> modelAssemblies = new List<IModelAssembly>();
                foreach (DMAssembly dmAssembly in this.Assemblies)
                {
                    modelAssemblies.Add(dmAssembly);
                }
                return modelAssemblies;
            }
        }

        #endregion

        public DMAssembly CreateAssembly(string assemblyName)
        {
            DMAssembly assembly = new DMAssembly(assemblyName);
            assembly.Solution = this;
            return assembly;
        }

        /// <summary>
        /// Find the first class that is mapped to the specified table name.
        /// </summary>
        /// <param name="mappedTableName">The table name that is being mapped to</param>
        /// <returns>the class that is mapped to the table. Null if not match found</returns>
        public DMClass FindClassByMappedTableName(string mappedTableName)
        {
            BusinessObjectCollection<DMClass> allClasses = AllClasses;
            if (allClasses.Count == 0) return null;
            return allClasses.Find(dmClass => string.Compare(dmClass.MappedTableName, mappedTableName, true) == 0);
        }

        public virtual DMAssembly DefaultAssembly
        {
            get { return Relationships.GetRelatedObject<DMAssembly>("DefaultAssembly"); }
            set
            {
                Relationships.SetRelatedObject("DefaultAssembly", value);
                if(value != null) Assemblies.Add(value);
            }
        }

        public UIGridColumnControlType GetGridColumnControlType(string assemblyName, string columnTypeName)
        {
            UIGridColumnControlType foundPropertyType = this.UIGridColumnControlTypes.Find
                (type => type.AssemblyName == assemblyName && type.TypeName == columnTypeName);
            return foundPropertyType;
        }

        public UIControlType GetUIFormFieldControlType(string assemblyName, string typeName)
        {
            UIControlType foundPropertyType = this.UIControlTypes.Find
                (type => type.AssemblyName == assemblyName && type.TypeName == typeName);
            return foundPropertyType;
        }

        public UITypeDefinition GetUIFilterControlType(string assemblyName, string typeName)
        {
            UITypeDefinition foundPropertyType = this.UIFilterControlTypes.Find
                (type => type.AssemblyName == assemblyName && type.TypeName == typeName);
            return foundPropertyType;
        }

        public DMPropertyType GetDMPropertyType(string assemblyName, string typeName)
        {
            DMPropertyType foundPropertyType = this.PropertyTypes.Find
                (type => type.AssemblyName == assemblyName && type.PropertyTypeName == typeName);
            return foundPropertyType;
        }

        public DMPropertyType GetDMPropertyType(string fullTypeName)
        {
            DMPropertyType foundPropertyType = this.PropertyTypes.Find
                (type => type.PropertyTypeFullName == fullTypeName);
            return foundPropertyType;
        }

        public DMPropertyType GetDMPropertyType(Guid? typeID)
        {
            DMPropertyType foundPropertyType = this.PropertyTypes.Find(type => type.PropertyTypeID == typeID);
            return foundPropertyType;
        }

        public UIControlMapperType GetUIControlMapperType(string assemblyName, string typeName)
        {
            UIControlMapperType foundControlMapperType = this.UIControlMapperTypes.Find
                (type => type.AssemblyName == assemblyName && type.TypeName == typeName);
            return foundControlMapperType;
        }

        public IList<IBOError> Validate()
        {
            List<IBOError> errors = new List<IBOError>();
            BusinessObjectCollection<DMClass> classes = this.AllClasses;
            IList<IBOError> tmpErrors;
            this.Status.HasWarnings(out tmpErrors);
            AddErrorList(tmpErrors, errors);
            foreach (DMAssembly dmAssembly in this.Assemblies)
            {
                dmAssembly.Status.HasWarnings(out tmpErrors);
                //if (!warnings) continue;
                AddErrorList(tmpErrors, errors);
            }
            foreach (DMClass dmClass in classes)
            {
                dmClass.Status.HasWarnings(out tmpErrors);
                //if (!warnings) continue;
                AddErrorList(tmpErrors, errors);
                foreach (DMRelationship relationship in dmClass.OwnerRelationships.ToArray())
                {
                    foreach (var relProp in relationship.RelationshipProperties)
                    {
                        relProp.Status.HasWarnings(out tmpErrors);
                        AddErrorList(tmpErrors, errors);
                    }
                    relationship.Status.HasWarnings(out tmpErrors);
                    AddErrorList(tmpErrors, errors);
                }
                foreach (DMProperty property in dmClass.Properties.ToArray())
                {
                    DMLookupListBusinessObject dmLookupList = property.LookupList as DMLookupListBusinessObject;
                    if (dmLookupList == null)
                    {
                        DMLookupListSimple dmLookupListSimple = property.LookupList as DMLookupListSimple;
                        if (dmLookupListSimple != null)
                        {
                            dmLookupListSimple.Status.HasWarnings(out tmpErrors);
                            AddErrorList(tmpErrors, errors);
                        }
                    }
                    else
                    {
                        dmLookupList.Status.HasWarnings(out tmpErrors);
                        AddErrorList(tmpErrors, errors);
                    }
                    property.Status.HasWarnings(out tmpErrors);
                    AddErrorList(tmpErrors, errors);
                }

                if (dmClass.ObjectIdentity != null)
                {
                    dmClass.ObjectIdentity.Status.HasWarnings(out tmpErrors);
                    AddErrorList(tmpErrors, errors);
                }

                foreach (DMUniqueConstraint constraint in dmClass.UniqueConstraints)
                {
                    constraint.Status.HasWarnings(out tmpErrors);
                    AddErrorList(tmpErrors, errors);
                }
                foreach (UIView uiView in dmClass.UIViews)
                {
                    uiView.Status.HasWarnings(out tmpErrors);
                    AddErrorList(tmpErrors, errors);
                }

                if (dmClass.InheritanceSuperClassRelationship != null)
                {
                    dmClass.InheritanceSuperClassRelationship.Status.HasWarnings(out tmpErrors);
                    AddErrorList(tmpErrors, errors);
                }
            }
            return errors;
        }

        private void AddErrorList(IList<IBOError> tmpErrors, List<IBOError> errors)
        {
            foreach (IBOError error in tmpErrors)
            {
                errors.Add(error);
            }
        }

        #region Implementation of IMergableBo
       
        ChangeType IMergableBo.PendingChangeType
        {
            get;
            set;
        }

        bool IMergableBo.PendingChangeApproved
        {
            get { return true; }
            set {  }
        }

        #endregion
    }
}