using System;
using FireStarterModeller.BO;
//using FireStarterModeller.Logic.LGMISServiceReference;
using Habanero.Base;
using Habanero.BO;

namespace FireStarterModeller.Logic
{
    public class ClassDefExporter
    {
        /// <summary>
        /// Provides a delegate to Update the BusinessObject with appropriate values during publishing
        /// </summary>
        public delegate void UpdateBusinessObjectStatus(IBusinessObject businessObject);

        private UpdateBusinessObjectStatus _updateBOStatusDelegate; //= this.UpdateBoStatusToNew;

        public void ExportSolution(DMSolution solution)
        {
            foreach (DMClass dmClass in solution.AllClasses)
            {
                this.CreateNewUIForm(dmClass);
            }
            string xmlForSolution = new DMToEntityClassDefsXmlConverter().ConvertToXml(solution);
//            LGMISWebServiceSoapClient lgmisServiceClient = new LGMISWebServiceSoapClient();
//            lgmisServiceClient.ReloadClassDefs(solution.SolutionName, xmlForSolution);
        }

        public void UpdateBOStatus(DMSolution solution, UpdateBusinessObjectStatus updateBOStatusDelegate)
        {
            if (solution == null) throw new ArgumentNullException("solution");
            if (updateBOStatusDelegate == null) throw new ArgumentNullException("updateBOStatusDelegate");
            _updateBOStatusDelegate = updateBOStatusDelegate;
            _updateBOStatusDelegate(solution);
            UpdatePropertyTypes(solution);
            UpdateGridColumnControlTypes(solution);
            UpdateControlTypes(solution);
            UpdateFilterControlTypes(solution);
            UpdateUIControlMapperTypes(solution);
            UpdateModules(solution);
        }

        public static void UpdateBOAndGeneratedPropToDirty(IBusinessObject businessObject)
        {
            if (businessObject != null)
            {
                if (businessObject.Props.Contains("Generated"))
                {
                    IBOProp generatedProp = businessObject.Props["Generated"];
                    generatedProp.Value = true;
                }
            }
        }

        private void UpdatePropertyTypes(DMSolution solution)
        {
            if (solution == null) return;
            MultipleRelationship<DMPropertyType> propertyTypeRelationship =
                (MultipleRelationship<DMPropertyType>) solution.Relationships["PropertyTypes"];
            BusinessObjectCollection<DMPropertyType> propertyTypes =
                propertyTypeRelationship.CurrentBusinessObjectCollection;
            foreach (DMPropertyType propertyType in propertyTypes)
            {
                _updateBOStatusDelegate(propertyType);
            }
        }


        private void UpdateGridColumnControlTypes(DMSolution solution)
        {
            if (solution == null) return;
            MultipleRelationship<UIGridColumnControlType> uiGridColumnControlTypesRelationship =
                (MultipleRelationship<UIGridColumnControlType>) solution.Relationships["UIGridColumnControlTypes"];
            BusinessObjectCollection<UIGridColumnControlType> uiGridColumnControlTypes =
                uiGridColumnControlTypesRelationship.CurrentBusinessObjectCollection;
            foreach (UIGridColumnControlType uiGridColumnControlType in uiGridColumnControlTypes)
            {
                _updateBOStatusDelegate(uiGridColumnControlType);
            }
        }

        private void UpdateControlTypes(DMSolution solution)
        {
            if (solution == null) return;
            MultipleRelationship<UIControlType> uiControlTypesRelationship =
                (MultipleRelationship<UIControlType>) solution.Relationships["UIControlTypes"];
            BusinessObjectCollection<UIControlType> uiControlTypes =
                uiControlTypesRelationship.CurrentBusinessObjectCollection;
            foreach (UIControlType uiControlType in uiControlTypes)
            {
                _updateBOStatusDelegate(uiControlType);
            }
        }

        private void UpdateFilterControlTypes(DMSolution solution)
        {
            if (solution == null) return;
            MultipleRelationship<UITypeDefinition> uiTypeDefinitionRelationship =
                (MultipleRelationship<UITypeDefinition>) solution.Relationships["UIFilterControlTypes"];
            BusinessObjectCollection<UITypeDefinition> uiTypeDefinitions =
                uiTypeDefinitionRelationship.CurrentBusinessObjectCollection;
            foreach (UITypeDefinition uiTypeDefinition in uiTypeDefinitions)
            {
                _updateBOStatusDelegate(uiTypeDefinition);
            }
        }

        private void UpdateUIControlMapperTypes(DMSolution solution)
        {
            if (solution == null) return;
            MultipleRelationship<UIControlMapperType> controlMapperTypeRelationship =
                (MultipleRelationship<UIControlMapperType>) solution.Relationships["UIControlMapperTypes"];
            BusinessObjectCollection<UIControlMapperType> controlMapperTypes =
                controlMapperTypeRelationship.CurrentBusinessObjectCollection;
            foreach (UIControlMapperType controlMapperType in controlMapperTypes)
            {
                _updateBOStatusDelegate(controlMapperType);
            }
        }


        private void UpdateModules(DMSolution solution)
        {
            if (solution == null) return;
            MultipleRelationship<DMAssembly> moduleRelationship =
                (MultipleRelationship<DMAssembly>) solution.Relationships["Assemblies"];
            BusinessObjectCollection<DMAssembly> assemblies = moduleRelationship.CurrentBusinessObjectCollection;
            foreach (DMAssembly assembly in assemblies)
            {
                _updateBOStatusDelegate(assembly);
                UpdateClasses(assembly);
            }
        }

        private void UpdateClasses(DMAssembly assembly)
        {
            if (assembly == null) return;
            MultipleRelationship<DMClass> entityRelationship =
                (MultipleRelationship<DMClass>) assembly.Relationships["Classes"];
            BusinessObjectCollection<DMClass> classes = entityRelationship.CurrentBusinessObjectCollection;
            foreach (DMClass dmClass in classes)
            {
                _updateBOStatusDelegate(dmClass);
//                    dmClass.Generated = true;
                UpdateProperties(dmClass);
                SingleRelationship<DMObjectIdentity> relationship =
                    (SingleRelationship<DMObjectIdentity>) dmClass.Relationships["ObjectIdentity"];
                DMObjectIdentity objectIdentity = relationship.GetRelatedObject();
                if (objectIdentity != null)
                {
                    _updateBOStatusDelegate(objectIdentity);
//                        objectIdentity.Generated = true;
                }
                UpdateUniqueConstraints(dmClass);
                UpdateOwnerRelationship(dmClass);
                UpdateRelatedRelationship(dmClass);
                UpdateUIViews(dmClass);
            }
        }

        private void UpdateProperties(DMClass dmClass)
        {
            if (dmClass == null) return;
            MultipleRelationship<DMProperty> propertyRelationship =
                (MultipleRelationship<DMProperty>) dmClass.Relationships["Properties"];
            BusinessObjectCollection<DMProperty> properties = propertyRelationship.CurrentBusinessObjectCollection;
            foreach (DMProperty dmProperty in properties)
            {
                _updateBOStatusDelegate(dmProperty);
//                    dmProperty.Generated = true;
                UpdateDefaultPropRule(dmProperty);
            }
        }

        private void UpdateDefaultPropRule(DMProperty dmProperty)
        {
            if (dmProperty == null) return;
            SingleRelationship<DMPropRule> defaultPropRuleRelationship =
                (SingleRelationship<DMPropRule>) dmProperty.Relationships["DefaultPropRule"];
            DMPropRule propRule = defaultPropRuleRelationship.GetRelatedObject();
            _updateBOStatusDelegate(propRule);
            if (propRule != null)
            {
                MultipleRelationship<DMPropRuleParameter> relationship =
                    (MultipleRelationship<DMPropRuleParameter>) propRule.Relationships["PropRuleParameters"];
                foreach (DMPropRuleParameter propRuleParameter in relationship.CurrentBusinessObjectCollection)
                {
                    _updateBOStatusDelegate(propRuleParameter);
                }
            }
        }

        private void UpdateUniqueConstraints(DMClass dmClass)
        {
            if (dmClass == null) return;
            MultipleRelationship<DMUniqueConstraint> ucRelationship =
                (MultipleRelationship<DMUniqueConstraint>) dmClass.Relationships["UniqueConstraints"];
            BusinessObjectCollection<DMUniqueConstraint> dmUniqueConstraints =
                ucRelationship.CurrentBusinessObjectCollection;
            foreach (DMUniqueConstraint dmUniqueConstraint in dmUniqueConstraints)
            {
                _updateBOStatusDelegate(dmUniqueConstraint);
                UpdateUniqueConstraintProps(dmUniqueConstraint);
            }
        }

        private void UpdateUniqueConstraintProps(DMUniqueConstraint constraint)
        {
            if (constraint == null) return;
            MultipleRelationship<DMUniqueConstraintProperty> ucPropRelationship =
                (MultipleRelationship<DMUniqueConstraintProperty>)
                constraint.Relationships["UniqueConstraintProperties"];
            BusinessObjectCollection<DMUniqueConstraintProperty> dmUniqueConstraintProperties =
                ucPropRelationship.CurrentBusinessObjectCollection;
            foreach (DMUniqueConstraintProperty uniqueConstraintProperty in dmUniqueConstraintProperties)
            {
                _updateBOStatusDelegate(uniqueConstraintProperty);
            }
        }

        private void UpdateOwnerRelationship(DMClass dmClass)
        {
            if (dmClass == null) return;
            MultipleRelationship<DMRelationship> ownerMultipleRelationship =
                (MultipleRelationship<DMRelationship>) dmClass.Relationships["OwnerRelationships"];
            BusinessObjectCollection<DMRelationship> ownerRelationships =
                ownerMultipleRelationship.CurrentBusinessObjectCollection;
            foreach (DMRelationship ownerRelationship in ownerRelationships)
            {
                _updateBOStatusDelegate(ownerRelationship);
//                    ownerRelationship.Generated = true;
                UpdateRelationshipProps(ownerRelationship);
            }
        }

        private void UpdateRelatedRelationship(DMClass dmClass)
        {
            if (dmClass == null) return;
            MultipleRelationship<DMRelationship> relatedMultipleRelationship =
                (MultipleRelationship<DMRelationship>) dmClass.Relationships["RelatedRelationships"];
            BusinessObjectCollection<DMRelationship> relatedRelationships =
                relatedMultipleRelationship.CurrentBusinessObjectCollection;
            foreach (DMRelationship relationship in relatedRelationships)
            {
                _updateBOStatusDelegate(relationship);
//                    relationship.Generated = true;
                UpdateRelationshipProps(relationship);
            }
        }

        private void UpdateRelationshipProps(DMRelationship relationship)
        {
            if (relationship == null) return;
            MultipleRelationship<DMRelationshipProperty> relationshipPropertiesRelationship =
                (MultipleRelationship<DMRelationshipProperty>) relationship.Relationships["RelationshipProperties"];
            BusinessObjectCollection<DMRelationshipProperty> relationshipProperties =
                relationshipPropertiesRelationship.CurrentBusinessObjectCollection;
            foreach (DMRelationshipProperty relationshipProperty in relationshipProperties)
            {
                _updateBOStatusDelegate(relationshipProperty);
//                    relationshipProperty.Generated = true;
            }
        }

        private void UpdateUIViews(DMClass dmClass)
        {
            if (dmClass == null) return;
            MultipleRelationship<UIView> uiViewRelationship =
                (MultipleRelationship<UIView>) dmClass.Relationships["UIViews"];
            BusinessObjectCollection<UIView> uiViews = uiViewRelationship.CurrentBusinessObjectCollection;
            foreach (UIView uiView in uiViews)
            {
                _updateBOStatusDelegate(uiView);
                UpdateUIForm(uiView);
                UpdateUIGrid(uiView);
            }
        }

        private void UpdateUIGrid(UIView uiView)
        {
            if (uiView == null) return;
            SingleRelationship<UIGridInfo> relationship =
                (SingleRelationship<UIGridInfo>) uiView.Relationships["UIGridInfo"];
            UIGridInfo uiGridInfo = relationship.GetRelatedObject();
            _updateBOStatusDelegate(uiGridInfo);
            UpdateUIColumns(uiGridInfo);
            UpdateUIGridFilter(uiGridInfo);
        }

        private void UpdateUIGridFilter(UIGridInfo uiGridInfo)
        {
            if (uiGridInfo == null) return;
            SingleRelationship<UIGridFilter> relationship =
                (SingleRelationship<UIGridFilter>) uiGridInfo.Relationships["UIGridFilter"];
            UIGridFilter uiGridFilter = relationship.GetRelatedObject();
            _updateBOStatusDelegate(uiGridFilter);
            UpdateUIGridFilterProps(uiGridFilter);
        }

        private void UpdateUIGridFilterProps(UIGridFilter uiGridFilter)
        {
            if (uiGridFilter == null) return;
            MultipleRelationship<UIGridFilterProp> relationship =
                (MultipleRelationship<UIGridFilterProp>) uiGridFilter.Relationships["UIGridFilterProps"];
            BusinessObjectCollection<UIGridFilterProp> gridFilterProps =
                relationship.CurrentBusinessObjectCollection;
            foreach (UIGridFilterProp uiGridFilterProp in gridFilterProps)
            {
                _updateBOStatusDelegate(uiGridFilterProp);
                MultipleRelationship<UIParameter> uiParameterRelationship =
                    (MultipleRelationship<UIParameter>) uiGridFilterProp.Relationships["UIParameters"];
                BusinessObjectCollection<UIParameter> uiParameters =
                    uiParameterRelationship.CurrentBusinessObjectCollection;
                UpdateUIParameters(uiParameters);
            }
        }

        private void UpdateUIColumns(UIGridInfo uiGridInfo)
        {
            if (uiGridInfo == null) return;
            MultipleRelationship<UIGridColumnInfo> relationship =
                (MultipleRelationship<UIGridColumnInfo>) uiGridInfo.Relationships["UIColumns"];
            BusinessObjectCollection<UIGridColumnInfo> gridColumnInfos =
                relationship.CurrentBusinessObjectCollection;
            foreach (UIGridColumnInfo uiGridColumnInfo in gridColumnInfos)
            {
                _updateBOStatusDelegate(uiGridColumnInfo);
                MultipleRelationship<UIParameter> uiParameterRelationship =
                    (MultipleRelationship<UIParameter>) uiGridColumnInfo.Relationships["UIParameters"];
                BusinessObjectCollection<UIParameter> uiParameters =
                    uiParameterRelationship.CurrentBusinessObjectCollection;
                UpdateUIParameters(uiParameters);
            }
        }

        private void UpdateUIParameters(BusinessObjectCollection<UIParameter> uiParameters)
        {
            foreach (UIParameter uiParameter in uiParameters)
            {
                _updateBOStatusDelegate(uiParameter);
            }
        }

        private void UpdateUIForm(UIView uiView)
        {
            if (uiView == null) return;
            SingleRelationship<UIFormInfo> uiFormRelationship =
                (SingleRelationship<UIFormInfo>) uiView.Relationships["UIFormInfo"];
            UIFormInfo uiFormInfo = uiFormRelationship.GetRelatedObject();
            _updateBOStatusDelegate(uiFormInfo);
            UpdateUITabs(uiFormInfo);
        }

        private void UpdateUITabs(UIFormInfo uiFormInfo)
        {
            if (uiFormInfo == null) return;
            MultipleRelationship<UITab> uiViewRelationship =
                (MultipleRelationship<UITab>) uiFormInfo.Relationships["UITabs"];
            BusinessObjectCollection<UITab> uiTabs = uiViewRelationship.CurrentBusinessObjectCollection;
            foreach (UITab uiTab in uiTabs)
            {
                _updateBOStatusDelegate(uiTab);
                UpdateUIColumnLayouts(uiTab);
            }
        }

        private void UpdateUIColumnLayouts(UITab uiTab)
        {
            if (uiTab == null) return;
            MultipleRelationship<UIColumnLayout> relationship =
                (MultipleRelationship<UIColumnLayout>) uiTab.Relationships["UIColumnLayouts"];
            BusinessObjectCollection<UIColumnLayout> uiColumnLayouts = relationship.CurrentBusinessObjectCollection;
            foreach (UIColumnLayout uiColumnLayout in uiColumnLayouts)
            {
                _updateBOStatusDelegate(uiColumnLayout);
                UpdateUIFields(uiColumnLayout);
            }
        }

        private void UpdateUIFields(UIColumnLayout columnLayout)
        {
            if (columnLayout == null) return;
            MultipleRelationship<UIField> relationship =
                (MultipleRelationship<UIField>) columnLayout.Relationships["UIFields"];
            BusinessObjectCollection<UIField> uiFields = relationship.CurrentBusinessObjectCollection;
            foreach (UIField uiField in uiFields)
            {
                _updateBOStatusDelegate(uiField);
            }
        }

        public void CreateNewUIForm(DMClass dmClass)
        {
            if (dmClass == null) throw new ArgumentNullException("dmClass");
            UIBOEditorCreator boEditorCreator = new UIBOEditorCreator();
            if (dmClass.UIViews.Count == 0)
            {
                //TODO: Throw some error
                return;
            }
            UIView view = dmClass.UIViews[0];
            UIFormInfo formInfo = view.UIFormInfo;
            ITransactionCommitter committer = BORegistry.DataAccessor.CreateTransactionCommitter();
            if (formInfo != null)
            {
                formInfo.MarkForDelete();
                committer.AddBusinessObject(formInfo);
            }
            UIFormInfo createdFormInfo = boEditorCreator.CreateBOEditor(dmClass);
            view.UIFormInfo = createdFormInfo;
            committer.AddBusinessObject(createdFormInfo);
            committer.CommitTransaction();
            createdFormInfo.UITabs.Refresh();
        }
    }
}