using System;
using System.Collections.Generic;
using System.Linq;
using FireStarterModeller.BO;

namespace FireStarterModeller.Logic.Wizards
{
    /// <summary>
    /// This is a controller class for renaming all aspects of a project to use standard naming conventions.
    /// I.e. It will rename the database table to be equal to the ClassName, the PKProp to be ClassName + ID.
    /// The FKProp to be RelationshipNameID etc.
    /// The DBFieldName to be PropName
    /// </summary>
    public class SolutionRenamer
    {

        public DMSolution Solution { get; private set; }

        public SolutionRenamer(DMSolution solution)
        {
            if (solution == null) throw new ArgumentNullException("solution");
            Solution = solution;
        }

/*        public IDMClass ClassDM { get; private set; }

        public IEnumerable<IDMRelationship> AllRelationshipWithRelatedFKs
        {
            get { return this.ClassDM.RelationshipsDM
                .Where(relationship => relationship.RelatedBOHasForeignKey)
                .Cast<IDMRelationship>(); }
        }

        public IList<IDMRelationship> RelationshipsWithFKToBeDeleted
        {
            get; private set;
        }

        public IEnumerable<DMLookupListBusinessObject> AllLookupLists
        {
            get { return this.ClassDM.DMLookupListBusinessObjects; }
        }

        public IList<DMLookupListBusinessObject> LookupListsWithPropsToBeDeleted
        {
            get; private set;
        }

        public SolutionRenamer(IDMClass dmClass)
        {
            if (dmClass == null) throw new ArgumentNullException("dmClass");
            this.ClassDM = dmClass;
            RelationshipsWithFKToBeDeleted = new List<IDMRelationship>();
            LookupListsWithPropsToBeDeleted = new List<DMLookupListBusinessObject>();
        }
        /// <summary>
        /// Deletes the DMClass and all its Related Business Objects
        /// </summary>
        public void Delete()
        {


            var relPropsToBeDeleted = this.RelationshipsWithFKToBeDeleted
                .SelectMany(relationship => relationship.RelationshipProperties);
            foreach (var relationshipProperty in relPropsToBeDeleted.ToList())
            {
                new DMRelationshipPropertyDeleter(relationshipProperty).Delete();
            }
            foreach (var dmRelationship in this.ClassDM.RelationshipsDM.ToArray())
            {
                dmRelationship.MarkForDelete();
            }
            var lookupPropsToBeDeleted = this.LookupListsWithPropsToBeDeleted
                .Select(o => o.DMProperty)
                .Where(prop => prop != null);
            foreach (var prop in lookupPropsToBeDeleted.ToList())
            {
                prop.MarkForDelete();
            }
            foreach (var lookupList in this.ClassDM.DMLookupListBusinessObjects.ToArray())
            {
                lookupList.MarkForDelete();
            }
            foreach (var uniqueConstraint in this.ClassDM.UniqueConstraints.ToArray())
            {
                uniqueConstraint.MarkForDelete();
            }
            this.ClassDM.MarkForDelete();
        }
        /// <summary>
        /// Generally Based on Some User Interface the user can select which 
        /// </summary>
        /// <param name="relationship"></param>
        public void AddRelationshipWithFKToBeDeleted(IDMRelationship relationship)
        {
            RelationshipsWithFKToBeDeleted.Add(relationship);
        }

        public void AddLookupListsWithPropsToBeDeleted(DMLookupListBusinessObject lookupList)
        {
            LookupListsWithPropsToBeDeleted.Add(lookupList);
        }

        public void MarkAllRelationshipsFKsToBeDeleted()
        {
            foreach (var relationship in AllRelationshipWithRelatedFKs)
            {
                this.AddRelationshipWithFKToBeDeleted(relationship);
            }
        }

        public void MarkAllLookupListsPropsToBeDeleted()
        {
            foreach (var lookupList in AllLookupLists)
            {
                this.AddLookupListsWithPropsToBeDeleted(lookupList);
            }
        }*/

        public void RenameToStandardNames()
        {
            this.Solution.AllClasses.ForEach(dmClass => dmClass.RenameClass());
        }
    }
}