using System;
using FireStarterModeller.BO;
using Habanero.Base;
using Habanero.Base.Exceptions;
using NUnit.Framework;

namespace FireStarterModeller.Test.Bo.Temp
{
    [TestFixture]
    public class TestDMProperty : TestBase
    {
        [Test]
        public void Test_GetParent()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = new DMClass();
            DMProperty dmProperty = new DMProperty();
            dmClass.Properties.Add(dmProperty);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IModelElement parent = ((IModelProperty) dmProperty).Parent;
            //---------------Test Result -----------------------
            Assert.AreSame(dmClass, parent);
        }


        [Test]
        public void Test_SetDMClass_ShouldCreateDefaultRule()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateUnsavedValidDMClass();
            DMProperty dmProperty = new DMProperty();

            //---------------Assert Precondition----------------
            Assert.IsNull(dmProperty.DefaultPropRule);
            //---------------Execute Test ----------------------

            dmProperty.Class = dmClass;
            //---------------Test Result -----------------------
            Assert.IsNotNull(dmProperty.DefaultPropRule);
        }

        [Test]
        public void Test_SetDMPropName_ShouldSetDefaultPropRuleName()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateUnsavedValidDMClass();
            DMProperty dmProperty = new DMProperty();
            dmProperty.Class = dmClass;
            //---------------Assert Precondition----------------
            Assert.IsNotNull(dmProperty.DefaultPropRule);
            Assert.IsNull(dmProperty.DefaultPropRule.PropRuleName);
            //---------------Execute Test ----------------------
            dmProperty.PropertyName = TestUtilsShared.GetRandomString();
            //---------------Test Result -----------------------
            Assert.IsNotNull(dmProperty.DefaultPropRule);
            Assert.AreEqual(dmProperty.PropertyName, dmProperty.DefaultPropRule.PropRuleName);
            Assert.IsNotNull(dmProperty.DefaultPropRule);
            Assert.IsNotNull(dmProperty.DefaultPropRule.PropRuleName);
        }

        [Test]
        public void Test_SetParentExceptionThrown()
        {
            //---------------Set up test pack-------------------
            IModelProperty modelProperty = new DMProperty();
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            Exception exceptionThrown = null;
            try
            {
                modelProperty.Parent = new DMClass();
            }
            catch (Exception ex)
            {
                exceptionThrown = ex;
            }
            //---------------Test Result -----------------------
            Assert.IsNotNull(exceptionThrown);
            Assert.IsInstanceOf(typeof (HabaneroDeveloperException), exceptionThrown);
            StringAssert.Contains
                ("Set the parent of the Property by setting the related Class object", exceptionThrown.Message);
        }

        [Test]
        public void Test_GetPropertyName()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = new DMProperty();
            string propertyName = TestUtils.GetRandomString();
            dmProperty.PropertyName = propertyName;
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string name = ((IModelProperty) dmProperty).PropertyName;
            //---------------Test Result -----------------------
            Assert.AreEqual(propertyName, name);
        }

        [Test]
        public void Test_SetPropertyName()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = new DMProperty();
            string propertyName = TestUtils.GetRandomString();

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            ((IModelProperty) dmProperty).PropertyName = propertyName;
            //---------------Test Result -----------------------
            Assert.AreEqual(propertyName, dmProperty.PropertyName);
        }

        [Test]
        public void Test_PropertyTypeString()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = new DMProperty();
            DMPropertyType dmPropertyType = new DMPropertyType();
            string assemblyName = TestUtils.GetRandomString();
            dmPropertyType.AssemblyName = assemblyName;
            string propertyTypeName = TestUtils.GetRandomString();
            dmPropertyType.PropertyTypeName = propertyTypeName;
            dmProperty.PropertyType = dmPropertyType;
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string name = ((IModelProperty) dmProperty).PropertyTypeString;
            //---------------Test Result -----------------------
            Assert.AreEqual(assemblyName + "." + propertyTypeName, name);
        }

#pragma warning disable 168
        [Test]
        public void Test_PropertyTypeString_NullPropertyType_ThrowsException()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = new DMProperty();
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            Exception exceptionThrown = null;
            try
            {
                string name = ((IModelProperty) dmProperty).PropertyTypeString;
            }
            catch (Exception ex)
            {
                exceptionThrown = ex;
            }
            //---------------Test Result -----------------------
            Assert.IsNotNull(exceptionThrown);
            Assert.IsInstanceOf(typeof (HabaneroDeveloperException), exceptionThrown);
            Assert.AreEqual
                (string.Format("The property type for the property '{0}' has not been set.", dmProperty.PropertyName),
                 exceptionThrown.Message);
        }
#pragma warning restore 168
        [Test]
        public void Test_HasSetter()
        {
            //---------------Set up test pack-------------------
            IModelProperty dmProperty = new DMProperty();
            dmProperty.HasSetter = true;
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            bool hasSetter = dmProperty.HasSetter;
            //---------------Test Result -----------------------
            Assert.IsTrue(hasSetter);
        }

        [Test]
        public void Test_DefaultValue()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = new DMProperty();
            string defaultValue = TestUtils.GetRandomString();
            dmProperty.DefaultValue = defaultValue;
            //---------------Assert Precondition----------------

            //---------------Execute Test --------------------
            string testDefaultValue = ((IModelProperty) dmProperty).DefaultValue;

            //---------------Test Result -----------------------
            Assert.AreEqual(defaultValue, testDefaultValue);
        }

        [Test]
        public void Test_IsCompulsory()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = new DMProperty();
            dmProperty.Compulsory = true;
            //---------------Assert Precondition----------------

            //---------------Execute Test --------------------
            bool isCompulsory = ((IModelProperty) dmProperty).IsCompulsory;

            //---------------Test Result -----------------------
            Assert.IsTrue(isCompulsory);
        }

        [Test]
        public void Test_PropertyType()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = new DMProperty();
            DMPropertyType dmPropertyType = new DMPropertyType();
            dmPropertyType.PropertyTypeName = "String";
            dmProperty.PropertyType = dmPropertyType;
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IModelPropertyType propertyType = ((IModelProperty) dmProperty).PropertyType;
            //---------------Test Result -----------------------
            Assert.AreSame(dmPropertyType, propertyType);
        }

        [Test]
        public void Test_ReadWriteRule()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = new DMProperty();
            dmProperty.ReadWriteRule = PropReadWriteRule.WriteNew;

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string readWriteRule = ((IModelProperty) dmProperty).ReadWriteRule;
            //---------------Test Result -----------------------
            Assert.AreSame(PropReadWriteRule.WriteNew.ToString(), readWriteRule);
        }
#pragma warning disable 168
        [Test]
        public void Test_ReadWriteRule_NullValue_ThrowsException()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = new DMProperty();
            dmProperty.ReadWriteRule = null;

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            Exception exceptionThrown = null;
            try
            {
                string readWriteRule = ((IModelProperty) dmProperty).ReadWriteRule;
            }
            catch (Exception ex)
            {
                exceptionThrown = ex;
            }
            //---------------Test Result -----------------------
            Assert.IsNotNull(exceptionThrown);
            Assert.IsInstanceOf(typeof (HabaneroDeveloperException), exceptionThrown);
            Assert.AreEqual
                (string.Format("ReadWriteRule has a null value for the property '{0}'", dmProperty.PropertyName),
                 exceptionThrown.Message);
        }
#pragma warning restore 168
//        [Test]
//        public void Test_Max_NoRule()
//        {
//            //---------------Set up test pack-------------------
//            DMProperty dmProperty = TestUtilsDMProperty.CreateSavedDMProperty();
//            dmProperty.PropertyTypeString = "System.String";
//            //---------------Assert Precondition----------------
//            Assert.AreEqual
//                ("System.String", dmProperty.PropertyTypeString,
//                 "This is messed up, and you can fix this test when you fix the messed up property types");
////            Assert.AreEqual(1, dmProperty.PropRules.Count);
////            DMPropRuleParameter defaultMaxLengthParameter = dmProperty.PropRules[0].PropRuleParameters[1];
//            Assert.AreEqual("maxLength", defaultMaxLengthParameter.KeyName);
//            Assert.IsNull(defaultMaxLengthParameter.Value);
//            //---------------Execute Test ----------------------
//            string max = ((IModelProperty) dmProperty).Max;
//            //---------------Test Result -----------------------
//            Assert.AreEqual("", max);
//        }

        [Test]
        public void Test_Max_EditDefaultRule()
        {
            //---------------Set up test pack-------------------
            const int maxValue = int.MaxValue - 100;
            DMProperty dmProperty = TestUtilsDMProperty.CreateSavedDMProperty();
            dmProperty.PropertyTypeString = "System.Int32";
            dmProperty.DefaultPropRule.PropRuleParameters[1].Value = maxValue.ToString();
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            string max = ((IModelProperty) dmProperty).Max;
            //---------------Test Result -----------------------
            Assert.AreEqual(maxValue.ToString(), max);
        }

        [Test]
        public void Test_SetPropertyTypeString_ShouldSetCorrectPropertyType()
        {
//            //---------------Set up test pack-------------------
//            DMSolution solution = TestUtilsDMSolution.
//            DMProperty dmProperty = new DMProperty();
//            //---------------Assert Precondition----------------
//
//            //---------------Execute Test ----------------------
//            dmProperty.PropertyTypeString ="System.Int32";
//            //---------------Test Result -----------------------
        }

        [Ignore(" TODO Temp commented out")] //TODO Brett 02 Apr 2009:
        [Test]
        public void Test_Max_Int()
        {
            Assert.Fail();
            //TODO 2009-03-09 Eric - when you change the property type, the default rules should change, like in the old Firestarter
//
//            //---------------Set up test pack-------------------
//            const int maxValue = int.MaxValue - 100;
//            DMProperty dmProperty = new DMProperty();
//            dmProperty.PropertyTypeID = "System.Int32"; //fix this
//
//            DMPropRule defaultPropRule = dmProperty.PropRules[0];
//            dmProperty.PropRules.Remove(defaultPropRule);
//            dmProperty.DefaultPropRule = null;
//
//            //---------------Assert Precondition----------------
//            Assert.AreEqual(0, dmProperty.PropRules.Count);
//            Assert.IsNull(dmProperty.DefaultPropRule);
//
//            //---------------Set up test pack-------------------
//            DMPropRule dmPropRule = new DMPropRule();
//            DMPropRuleParameter propRuleParameter = new DMPropRuleParameter();
//            propRuleParameter.KeyName = "max";
//            propRuleParameter.Value = maxValue.ToString();
//            propRuleParameter.PropRule = dmPropRule;
//            dmPropRule.Property = dmProperty;
//            //---------------Assert Precondition----------------
//            Assert.AreEqual("System.Int32", dmProperty.PropertyTypeID, "Fix me");
//            Assert.AreEqual(1, dmProperty.PropRules.Count);
//            //---------------Execute Test ----------------------
//            string max = ((IModelProperty)dmProperty).Max;
//            //---------------Test Result -----------------------
//            Assert.AreEqual(maxValue.ToString(), max);
        }

//        [Test,
//         Ignore(
//             "This is a fairly trivial case, get it done when there's time - basically, if you have multiple max's, must take the minimum of them, make similar rule for min"
//             )]
//        public void Test_Max_MultipleRulesGetsMinimumLength()
//        {
//            //---------------Set up test pack-------------------
//            string biggerMaxValue = (int.MaxValue - 100).ToString();
//            string smallerMaxValue = (int.MaxValue - 110).ToString();
//
//            DMProperty dmProperty = new DMProperty();
//
//            DMPropRule dmPropRule = new DMPropRule();
//            DMPropRuleParameter propRuleParameter = new DMPropRuleParameter();
//            propRuleParameter.KeyName = "maxLength";
//            propRuleParameter.Value = smallerMaxValue;
//            propRuleParameter.PropRule = dmPropRule;
//            dmPropRule.Property = dmProperty;
//
//            //---------------Assert Precondition----------------
//            Assert.AreEqual
//                ("System.String", dmProperty.PropertyTypeID,
//                 "This is messed up, and you can fix this test when you fix the messed up property types");
////            Assert.AreEqual(2, dmProperty.PropRules.Count);
////            DMPropRuleParameter defaultMaxLengthParameter = dmProperty.PropRules[0].PropRuleParameters[1];
////            Assert.AreEqual("maxLength", defaultMaxLengthParameter.KeyName);
//            //---------------Execute Test ----------------------
//            dmProperty.PropRules[0].PropRuleParameters[1].Value = biggerMaxValue;
//            string max = ((IModelProperty) dmProperty).Max;
//            //---------------Test Result -----------------------
//            Assert.AreEqual(smallerMaxValue, max);
//        }

//        [Test] //This test ensures that the foreach is working to find the rule at deeper levels
//        public void Test_Max_NewRule_WithNonDefaultParameterName()
//        {
//            //---------------Set up test pack-------------------
//            DMProperty dmProperty = new DMProperty();
//            DMPropRule defaultPropRule = dmProperty.DefaultPropRule;
//            dmProperty.DefaultPropRule = null;
//
//            //---------------Assert Precondition----------------
////            Assert.AreEqual(0, dmProperty.PropRules.Count);
//            Assert.IsNull(dmProperty.DefaultPropRule);
//
//            //---------------Set up test pack-------------------
//            DMPropRule emptyPropRule = new DMPropRule();
//            dmProperty.DefaultPropRule = emptyPropRule;
//
//            DMPropRule dmPropRule = new DMPropRule();
//            DMPropRuleParameter propRuleParameter = new DMPropRuleParameter();
//            const string ruleParamName = "maxLength";
//            propRuleParameter.KeyName = ruleParamName;
//            const string expectedValue = "passValue";
//            propRuleParameter.Value = expectedValue;
//            propRuleParameter.PropRule = dmPropRule;
//            dmProperty.PropRules.Add(dmPropRule);
//
//            //---------------Assert Precondition----------------
//            Assert.AreEqual(2, dmProperty.PropRules.Count);
//            Assert.AreEqual(0, emptyPropRule.PropRuleParameters.Count);
//            Assert.AreEqual(ruleParamName, dmProperty.PropRules[1].PropRuleParameters[0].KeyName);
//            //---------------Execute Test ----------------------
//            string max = ((IModelProperty) dmProperty).Max;
//            //---------------Test Result -----------------------
//            Assert.AreEqual(expectedValue, max);
//        }

//        [Test]
//        public void Test_Min_NoRule()
//        {
//            //---------------Set up test pack-------------------
//            DMProperty dmProperty = TestUtilsDMProperty.CreateSavedDMProperty();
//            dmProperty.PropertyTypeString = "System.String";
//            //---------------Assert Precondition----------------
//            Assert.AreEqual
//                ("System.String", dmProperty.PropertyTypeString,
//                 "This is messed up, and you can fix this test when you fix the messed up property types");
//            Assert.AreEqual(1, dmProperty.PropRules.Count);
//            DMPropRuleParameter defaultMaxLengthParameter = dmProperty.PropRules[0].PropRuleParameters[0];
//            Assert.AreEqual("minLength", defaultMaxLengthParameter.KeyName);
//            Assert.IsNull(defaultMaxLengthParameter.Value);
//            //---------------Execute Test ----------------------
//            string min = ((IModelProperty) dmProperty).Min;
//            //---------------Test Result -----------------------
//            Assert.AreEqual("", min);
//        }

//        [Test]
//        public void Test_MinLength_ForString_ShouldSetMinLength()
//        {
//            //---------------Set up test pack-------------------
//            const int minValue = 0 + 100;
//            DMProperty dmProperty = TestUtilsDMProperty.CreateSavedDMProperty();
//            dmProperty.PropertyTypeString = "System.Int32";
//            dmProperty.PropRules[0].PropRuleParameters[0].Value = minValue.ToString();
//            //---------------Assert Precondition----------------
//
//            //---------------Execute Test ----------------------
//            string min = ((IModelProperty) dmProperty).Min;
//            //---------------Test Result -----------------------
//            Assert.AreEqual(minValue.ToString(), min);
//        }

//        [Test]
//        public void Test_Min_Int()
//        {
//            //---------------Set up test pack-------------------
//            const int minValue = int.MinValue + 100;
//            DMProperty dmProperty = TestUtilsDMProperty.CreateUnsavedValidDMProperty();
//            dmProperty.PropertyTypeString = "System.Int32";
//
//            DMPropRule defaultPropRule = dmProperty.PropRules[0];
//            dmProperty.PropRules.Remove(defaultPropRule);
//            dmProperty.DefaultPropRule = null;
//
//            //---------------Assert Precondition----------------
//            Assert.AreEqual(0, dmProperty.PropRules.Count);
//            Assert.IsNull(dmProperty.DefaultPropRule);
//
//            //---------------Set up test pack-------------------
//            DMPropRule dmPropRule = new DMPropRule();
//            DMPropRuleParameter propRuleParameter = new DMPropRuleParameter();
//            propRuleParameter.KeyName = "min";
//            propRuleParameter.Value = minValue.ToString();
//            propRuleParameter.PropRule = dmPropRule;
//            dmPropRule.Property = dmProperty;
//            //---------------Assert Precondition----------------
//            Assert.AreEqual("System.Int32", dmProperty.PropertyTypeString);
//            Assert.AreEqual(1, dmProperty.PropRules.Count);
//            //---------------Execute Test ----------------------
//            string min = ((IModelProperty) dmProperty).Min;
//            //---------------Test Result -----------------------
//            Assert.AreEqual(minValue.ToString(), min);
//        }

    }
}