using System;
using System.Collections.Generic;
using System.Linq;
using FireStarter.Base;
using FireStarterModeller.BO;
using Habanero.Base.Exceptions;
using Habanero.BO;
using NUnit.Framework;

namespace FireStarterModeller.Test.Bo.Temp
{
    [TestFixture]
    public class TestDMSolution : TestBase
    {
        [Test]
        public void Test_SetParentExceptionThrown()
        {
            //---------------Set up test pack-------------------
            IModelSolution solution = new DMSolution();

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            Exception exceptionThrown = null;
            try
            {
                solution.Parent = new DMSolution();
            }
            catch (Exception ex)
            {
                exceptionThrown = ex;
            }
            //---------------Test Result -----------------------
            Assert.IsNotNull(exceptionThrown);
            Assert.IsInstanceOf(typeof (HabaneroDeveloperException), exceptionThrown);
            StringAssert.Contains("A solution cannot have a parent", exceptionThrown.Message);
        }


        [Test]
        public void Test_GetParentIsNull()
        {
            //---------------Set up test pack-------------------


            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IModelSolution solution = new DMSolution();
            //---------------Test Result -----------------------
            Assert.IsNull(solution.Parent);
        }

        [Test]
        public void Test_GetAllModelClasses()
        {
            //---------------Set up test pack-------------------
            IModelSolution solution = new DMSolution();
            ((DMSolution) solution).Assemblies.Add(new DMAssembly(TestUtils.GetRandomString()));
            DMClass dmClass1 = new DMClass();
            DMClass dmClass2 = new DMClass();
            DMClass dmClass3 = new DMClass();

            //---------------Assert Precondition----------------
            Assert.AreEqual(0, solution.ModelClasses.Count());
            //---------------Execute Test ----------------------
            BusinessObjectCollection<DMClass> classes = ((DMSolution) solution).Assemblies[0].Classes;
            classes.Add(dmClass1);
            classes.Add(dmClass2);
            classes.Add(dmClass3);

            //---------------Test Result -----------------------
            Assert.AreEqual(3, classes.Count);
            Assert.AreEqual(3, solution.ModelClasses.Count());
            Assert.AreSame(dmClass1, solution.ModelClasses[0]);
            Assert.AreSame(dmClass2, solution.ModelClasses[1]);
            Assert.AreSame(dmClass3, solution.ModelClasses[2]);
        }

        [Test]
        public void Test_SetAssmeblies()
        {
            //---------------Set up test pack-------------------
            DMSolution solution = new DMSolution();
            DMAssembly assembly = new DMAssembly(TestUtils.GetRandomString());
            solution.Assemblies.Add(assembly);

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IList<IModelAssembly> assemblies = ((IModelSolution) solution).ModelAssemblies;
            //---------------Test Result -----------------------
            Assert.AreEqual(1, assemblies.Count);
            Assert.AreSame(assembly, assemblies[0]);
        }


        //Testing of project name properties will be generated by test generation
    }
}