using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using FireStarterModeller.UI.DetailsView.WPF;
using Habanero.Util;
using NUnit.Framework;

namespace FireStarterModeller.Test.UI.DetailsView.WPF
{
    [TestFixture]
    public class TestNewItemPopup
    {
        [Test]
        public void TestOKButtonClickedEvent()
        {
            CrossThreadRunner runner = new CrossThreadRunner();
            runner.RunInSTA
                (() =>
                 {
                     //---------------Set up test pack-------------------
                     RenameItemPopup popup = new RenameItemPopup();
                     Panel panel = (Panel) popup.Child;
                     Button button = GetOKButton(panel);
                     RoutedEventArgs clickEventArgs = new RoutedEventArgs(ButtonBase.ClickEvent);
                     bool eventRaised = false;
                     popup.OKButtonClicked += (sender, e) => eventRaised = true;

                     //---------------Execute Test ----------------------
                     button.RaiseEvent(clickEventArgs);
                     //---------------Test Result -----------------------
                     Assert.IsTrue(eventRaised);
                     //---------------Tear Down -------------------------      
                 });
        }

        [Test]
        public void TestInit()
        {
            CrossThreadRunner runner = new CrossThreadRunner();
            runner.RunInSTA
                (() =>
                 {
                     //---------------Set up test pack-------------------
                     NewItemPopup popup = new NewItemPopup();
                     Panel panel = (Panel) popup.Child;
                     const string itemType = "assembly";
                     const string itemName = "assembly1";
                     //---------------Execute Test ----------------------
                     popup.Init(itemType);
                     string labelContent = GetLabelContent(panel);
                     TextBox nameTextBox = GetTextBox(panel);
                     nameTextBox.Text = itemName;
                     //---------------Test Result -----------------------
                     Assert.AreEqual("Create new assembly called...", labelContent);
                     Assert.AreEqual(itemName, popup.NewItemName);
                     //---------------Tear Down -------------------------      
                 });
        }

        private string GetLabelContent(Panel panel)
        {
            foreach (object child in panel.Children)
            {
                if (child is Label) return (string) ((Label) child).Content;
            }
            return null;
        }

        private TextBox GetTextBox(Panel panel)
        {
            foreach (object child in panel.Children)
            {
                if (child is TextBox) return (TextBox) child;
            }
            return null;
        }

        private Button GetOKButton(Panel panel)
        {
            foreach (object child in panel.Children)
            {
                if (child is Button) return (Button) child;
            }
            return null;
        }
    }
}