using System.Windows.Forms;
using FireStarterModeller.UI;
using Habanero.Faces.Base;
using Habanero.Faces.Win;
using NUnit.Framework;

namespace FireStarterModeller.Test.UI.Mapper
{
    [TestFixture]
    public class TestExtendedTextBox
    {
        private static IControlFactory GetControlFactory()
        {
            return new ControlFactoryWin();
        }

        [Test]
        public void Test_Layout()
        {
            //--------------- Set up test pack-------------------
            IControlFactory controlFactory = GetControlFactory();
            //--------------- Test Preconditions ----------------

            //--------------- Execute Test ----------------------
            ExtendedTextBoxWin extendedTextBox = new ExtendedTextBoxWin(controlFactory);
            //--------------- Test Result -----------------------
            Assert.AreEqual(2, extendedTextBox.Controls.Count);
            Control control1 = extendedTextBox.Controls[0];
            Control control2 = extendedTextBox.Controls[1];
            Assert.IsInstanceOf(typeof(ITextBox), control1);
            Assert.IsInstanceOf(typeof(IButton), control2);
            Assert.AreEqual("...", control2.Text);
            Assert.AreEqual(0, control1.Left);
            Assert.LessOrEqual(control1.Width, control2.Left);
            Assert.GreaterOrEqual(extendedTextBox.Width, control2.Left + control2.Width);
        }

        [Test]
        public void Test_GetTextBox()
        {
            //--------------- Set up test pack ------------------
            IControlFactory controlFactory = GetControlFactory();
            ExtendedTextBoxWin extendedTextBox = new ExtendedTextBoxWin(controlFactory);
            //--------------- Test Preconditions ----------------

            //--------------- Execute Test ----------------------
            ITextBox textBox = extendedTextBox.TextBox;
            //--------------- Test Result -----------------------
            Assert.IsNotNull(textBox);
        }

        [Test]
        public void Test_GetButton()
        {
            //--------------- Set up test pack ------------------
            IControlFactory controlFactory = GetControlFactory();
            ExtendedTextBoxWin extendedTextBox = new ExtendedTextBoxWin(controlFactory);
            //--------------- Test Preconditions ----------------

            //--------------- Execute Test ----------------------
            IButton button = extendedTextBox.Button;
            //--------------- Test Result -----------------------
            Assert.IsNotNull(button);
        }
    }
}