using System.Collections.Generic;
using System.Windows.Forms;
using FireStarterModeller.BO;
using FireStarterModeller.Test.BO;
using FireStarterModeller.UI;
using Habanero.UI.Base;
using Habanero.UI.Win;
using NUnit.Framework;
using DockStyle=Habanero.UI.Base.DockStyle;

namespace FireStarterModeller.Test.UI
{
    [TestFixture]
    public class TestDMClassDetailCollapsiblePanelGroupControl
    {
        [TestFixtureSetUp]
        public void SetupFixture()
        {
            UITestUtils.SetupFixture();
        }


        [SetUp]
        public virtual void SetupTest()
        {
            UITestUtils.SetupTest();

        }

        [Test]
        public void Test_CreateControl()
        {
            //---------------Set up test pack-------------------

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            DMClassDetailCollapsiblePanelGroupControl control = new DMClassDetailCollapsiblePanelGroupControl();
            //---------------Test Result -----------------------
            Assert.IsNotNull(control);
            Assert.IsNotNull(control.ControlFactory);
            Assert.AreSame(GlobalUIRegistry.ControlFactory, control.ControlFactory);
            Assert.AreEqual(1, control.Controls.Count);
            Assert.IsInstanceOfType(typeof(PanelWin), control.Controls[0]);
            PanelWin panelWin = (PanelWin) control.Controls[0];
            Assert.IsTrue(panelWin.AutoScroll);
            Assert.AreEqual(1, panelWin.Controls.Count);
            Assert.IsInstanceOfType(typeof(CollapsiblePanelGroupControlWin), panelWin.Controls[0]);
            
        }

        [Test]
        public void Test_HasClassInfoDetailsCollapsiblePanel()
        {
            //---------------Set up test pack-------------------
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();

            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            List<ICollapsiblePanel> collapsiblePanels = panelGroupControl.PanelList;
            //---------------Test Result -----------------------
            Assert.IsNotNull(collapsiblePanels);
            Assert.AreEqual(4, collapsiblePanels.Count);
            Assert.IsInstanceOfType(typeof(DMClassEditorControl), collapsiblePanels[0].ContentControl);
            Assert.AreEqual("Entity Details", collapsiblePanels[0].CollapseButton.Text);
        }

        [Test]
        public void Test_HasPropertyInfoCollapsiblePanel()
        {
            //---------------Set up test pack-------------------
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();

            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            List<ICollapsiblePanel> collapsiblePanels = panelGroupControl.PanelList;
            //---------------Test Result -----------------------
            Assert.IsNotNull(collapsiblePanels);
            Assert.AreEqual(4, collapsiblePanels.Count);
            Assert.IsInstanceOfType(typeof(DMPropertyAndRuleEditorControl), collapsiblePanels[1].ContentControl);
            Assert.AreEqual("Properties", collapsiblePanels[1].CollapseButton.Text);
        }

        [Test]
        public void Test_HasRelationshipInfoCollapsiblePanel()
        {
            //---------------Set up test pack-------------------
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();

            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            List<ICollapsiblePanel> collapsiblePanels = panelGroupControl.PanelList;
            //---------------Test Result -----------------------
            Assert.IsNotNull(collapsiblePanels);
            Assert.AreEqual(4, collapsiblePanels.Count);
            Assert.IsInstanceOfType(typeof(DMRelationshipsControl), collapsiblePanels[2].ContentControl);
            Assert.AreEqual("Relationships", collapsiblePanels[2].CollapseButton.Text);

        }

        [Test]
        public void Test_HasUniqueConstraintsCollapsiblePanel()
        {
            //---------------Set up test pack-------------------
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();

            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            List<ICollapsiblePanel> collapsiblePanels = panelGroupControl.PanelList;
            //---------------Test Result -----------------------
            Assert.IsNotNull(collapsiblePanels);
            Assert.AreEqual(4, collapsiblePanels.Count);
            Assert.IsInstanceOfType(typeof(DMUniqueConstraintAndPropertyEditorControl), collapsiblePanels[3].ContentControl);
            Assert.AreEqual("Unique Constraints", collapsiblePanels[3].CollapseButton.Text);

        }

        [Test]
        public void Test_HasClassDetailAndPropertiesPanelsPinned()
        {
            //---------------Set up test pack-------------------
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();
            List<ICollapsiblePanel> collapsiblePanels = panelGroupControl.PanelList;
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            ICollapsiblePanel classDetailPanel = collapsiblePanels[0];
            ICollapsiblePanel propertiesPanel = collapsiblePanels[1];
            //---------------Test Result -----------------------
            Assert.IsTrue(classDetailPanel.Pinned);
            Assert.IsTrue(propertiesPanel.Pinned);
            Assert.IsFalse(collapsiblePanels[2].Pinned);
            Assert.IsFalse(collapsiblePanels[3].Pinned);

        }

        [Test]
        public void TestDmClassSelected_SetsBusinessObjectOnBusinessObjectControl()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClass();
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();

            //---------------Execute Test ----------------------
            panelGroupControl.DMClass = dmClass;
            //---------------Test Result -----------------------
            Assert.AreSame(dmClass, panelGroupControl.DMClassEditorControl.BusinessObject);
        }

        [Test]
        public void TestDmClassSelected_SetsBusinessObjectOnPropertiesControl()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClass();
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();

            //---------------Execute Test ----------------------
            panelGroupControl.DMClass = dmClass;
            //---------------Test Result -----------------------
            Assert.AreSame(dmClass, panelGroupControl.DMPropertyAndRuleEditorControl.Class);
        }

        [Test]
        public void TestBusinessObjectSelected_SetsBusinessObjectOnRelationshipsControl()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClass();
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();

            //---------------Execute Test ----------------------
            panelGroupControl.DMClass = dmClass;
            //---------------Test Result -----------------------
            Assert.AreSame(dmClass, panelGroupControl.DMRelationshipsControl.Class);
        }

        [Test]
        public void TestBusinessObjectSelected_SetsBusinessObjectOnUniqueConstraintsControl()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClass();
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();

            //---------------Execute Test ----------------------
            panelGroupControl.DMClass = dmClass;
            //---------------Test Result -----------------------
            Assert.AreSame(dmClass, panelGroupControl.UniqueConstraintPropertyEditorControl.Class);

        }

        [Test]
        public void Test_SetSolution_ShouldSetSolutionInRelationshipControl()
        {
            //---------------Set up test pack-------------------
            DMSolution dmSolution = TestUtilsShared.CreateUnsavedValidDMSolution();
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();

            //---------------Assert Precondition----------------
            Assert.IsNull(panelGroupControl.Solution);
            //---------------Execute Test ----------------------
            panelGroupControl.Solution = dmSolution;
            //---------------Test Result -----------------------
            DMRelationshipSelectorControl selector = (DMRelationshipSelectorControl) panelGroupControl.DMRelationshipsControl.RelationshipSelector;
            Assert.AreSame(dmSolution, selector.Solution);
        }


        [Test]
        public void Test_ResetSolution_WithNoClasses_ShouldSetDMClassInPanelToNull()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClass();
            DMSolution solutionNoClasses = TestUtilsShared.CreateUnsavedValidDMSolution();
            DMSolution dmSolution = dmClass.Solution;
            DMClassDetailCollapsiblePanelGroupControl panelGroupControl = new DMClassDetailCollapsiblePanelGroupControl();
            DMClassEditorControl classEditorControl = panelGroupControl.DMClassEditorControl;
            panelGroupControl.Solution = dmSolution;
            panelGroupControl.DMClass = dmClass;
            //---------------Assert Precondition----------------
            Assert.AreSame(dmSolution, panelGroupControl.Solution);
            Assert.AreSame(dmClass, panelGroupControl.DMClass);
            Assert.AreSame(dmClass, classEditorControl.BusinessObject);
            Assert.IsTrue(classEditorControl.Enabled);
            Assert.AreEqual(0, solutionNoClasses.AllClasses.Count);
            //---------------Execute Test ----------------------
            panelGroupControl.Solution = solutionNoClasses;
            //---------------Test Result -----------------------
            Assert.AreSame(solutionNoClasses, panelGroupControl.Solution);
            Assert.IsNull(panelGroupControl.DMClass);
            Assert.IsNull(classEditorControl.BusinessObject);
        }
    }

}
