using FireStarterModeller.BO;
using FireStarterModeller.Test.BO;
using FireStarterModeller.UI;
using Habanero.Base.Exceptions;
using Habanero.Faces.Win;
using NUnit.Framework;

namespace FireStarterModeller.Test.UI
{
    [TestFixture]
    public class TestDMUniqueConstraintColEditorControl
    {

        [TestFixtureSetUp]
        public void SetupFixture()
        {
            UITestUtils.SetupFixture();
        }


        [SetUp]
        public virtual void SetupTest()
        {
            UITestUtils.SetupTest();

        }

        [Test]
        public void Test_Construct_NullControlFactory_ShouldRaiseAppropriateError()
        {
            //---------------Set up test pack-------------------
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            try
            {
                new DMUniqueConstraintColEditorControl(null);
                Assert.Fail("expected ArgumentNullException");
            }
                //---------------Test Result -----------------------
            catch (HabaneroArgumentException ex)
            {
                StringAssert.Contains("The argument 'controlFactory' is not valid", ex.Message);
                StringAssert.Contains("Cannot create an 'DMUniqueConstraintSelectorControl' if the control factory is null", ex.Message);
            }
        }

        [Test]
        public void Test_Construct()
        {
            //---------------Set up test pack-------------------
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            DMUniqueConstraintColEditorControl control = new DMUniqueConstraintColEditorControl(GetControlFactory());
            //---------------Test Result -----------------------
            Assert.IsNotNull(control.GridControl);
            Assert.IsNotNull(control.UniqueConstraintSelectorControl);
            Assert.IsNotNull(control.UniqueConstraintEditorControl);
            Assert.AreEqual(3, control.Controls.Count, "Two controls and the splitter");
            Assert.IsFalse(control.UniqueConstraintSelectorControl.Enabled);
            Assert.IsFalse(control.Enabled);
        }

        [Test]
        public void Test_SetBusinessObject_SetsUpUniqueConstraints()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClassWithUniqueConstraints();
            DMUniqueConstraintColEditorControl control =
                new DMUniqueConstraintColEditorControl(GetControlFactory());
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            control.BusinessObject = dmClass;
            //---------------Test Result -----------------------
            Assert.AreSame(dmClass, control.Class);
            Assert.AreSame(dmClass, control.BusinessObject);
            Assert.AreEqual(dmClass.UniqueConstraints.Count, control.UniqueConstraints.Count);
            Assert.AreSame(dmClass, control.UniqueConstraintSelectorControl.BusinessObject);
            Assert.IsTrue(control.UniqueConstraintSelectorControl.Enabled);
            Assert.IsTrue(control.Enabled);
        }
        [Test]
        public void Test_SetDMClass_SetsUpUniqueConstraints()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClassWithUniqueConstraints();
            DMUniqueConstraintColEditorControl control =
                new DMUniqueConstraintColEditorControl(GetControlFactory());
            //---------------Assert Precondition---------------- 

            //---------------Execute Test ----------------------
            control.Class = dmClass;
            //---------------Test Result -----------------------
            Assert.AreSame(dmClass, control.Class);
            Assert.AreEqual(dmClass.UniqueConstraints.Count, control.UniqueConstraints.Count);
            Assert.AreEqual(dmClass.UniqueConstraints.Count, control.UniqueConstraints.Count);
            Assert.AreSame(dmClass, control.UniqueConstraintSelectorControl.BusinessObject);

        }

        [Test]
        public void TestDisabledAtCreation()
        {
            //---------------Execute Test ----------------------
            DMUniqueConstraintColEditorControl control =
                new DMUniqueConstraintColEditorControl(GetControlFactory());
            //---------------Test Result -----------------------

            Assert.IsFalse(control.Enabled);
        }

        [Test]
        public void TestSetDMClassEnables()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClassWithUniqueConstraints();
            DMUniqueConstraintColEditorControl control =
                new DMUniqueConstraintColEditorControl(GetControlFactory());
      
            //---------------Execute Test ----------------------
            control.Class = dmClass;
            //---------------Test Result -----------------------
            Assert.IsTrue(control.Enabled);
        }

        [Test]
        public void Test_SelectUniqueConstraint_SetsUniqueConstraintInEditorControl()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClassWithUniqueConstraints();
            DMUniqueConstraintColEditorControl control =
                new DMUniqueConstraintColEditorControl(GetControlFactory());
            control.Class = dmClass;
            //----------------Assert Preconditions---------------
            Assert.IsTrue(dmClass.UniqueConstraints.Count >1);
            //---------------Execute Test ----------------------
            control.SelectedBusinessObject = dmClass.UniqueConstraints[1];
            //---------------Test Result -----------------------
            Assert.IsNotNull(control.UniqueConstraintEditorControl.BusinessObject);
        }
        [Test]
        public void TestSetToNullDisables()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClassWithUniqueConstraints();
            DMUniqueConstraintColEditorControl control =
                new DMUniqueConstraintColEditorControl(GetControlFactory()) { Class = dmClass};
            //----------------Assert Preconditions---------------
            Assert.IsTrue(control.Enabled);  
            //---------------Execute Test ----------------------
            control.Class = null;
            //---------------Test Result -----------------------
            Assert.IsFalse(control.Enabled);  
        }

        [Test]
        public void TestSetUniqueContraint()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsShared.CreateSavedDMClass();
            TestUtilsDMUniqueConstraint.CreateUnsavedValidDMUniqueConstraint(dmClass);

            DMUniqueConstraintColEditorControl control = new DMUniqueConstraintColEditorControl(GetControlFactory()) {Class = dmClass};
                       
            //----------------Assert Preconditions---------------
            Assert.IsTrue(dmClass.UniqueConstraints.Count > 0);
            //---------------Execute Test ----------------------
            DMUniqueConstraint dmUniqueConstraint = dmClass.UniqueConstraints[0];
            control.UniqueConstraint = dmUniqueConstraint;
            //---------------Test Result -----------------------
            Assert.AreSame(dmUniqueConstraint, control.UniqueConstraint);
            Assert.AreSame(dmUniqueConstraint, control.SelectedBusinessObject);
        }

        private static ControlFactoryWin GetControlFactory()
        {
            return new ControlFactoryWin();
        }

//        [Test]
//        public void TestLinkBetweenDMPropertyAndPropRuleInfoControls()
//        {
//            //---------------Set up test pack-------------------
//            DMClass bo = TestUtilsShared.CreateSavedDMClassWithProps();
//            TestUtilsShared.CreateSavedDMProperty(bo);
//
//            DMPropertyAndRuleEditorControl control = new DMPropertyAndRuleEditorControl(new ControlFactoryWin());
//            control.Class = bo;
//
//            //---------------Execute Test ----------------------
//            control.Property = bo.Properties[0];
//            //---------------Test Result -----------------------
//            Assert.AreSame(control.Property, control.DMUniqueConstraintPropertySelectorControl.Property);
//            //---------------Tear Down -------------------------          
//        }


    }
}