using System.Collections.Generic;
using FireStarterModeller.BO;
using FireStarterModeller.Test.BO;
using FireStarterModeller.UI;
using Habanero.Base.Exceptions;
using Habanero.BO;
using Habanero.Faces.Base;
using NUnit.Framework;

namespace FireStarterModeller.Test.UI
{
    [TestFixture]
    public class TestCreateGridWizardController
    {
        //protected static readonly DatabaseConfig _databaseConfig = new DatabaseConfig
        //    (DatabaseConfig.SqlServer, "localhost", "lgta_LGMIS_Definition_test", "sa", "sa", "1000");
        [SetUp]
        public void SetupTest()
        {
            //Runs every time that any testmethod is executed
            UITestUtils.SetupTest();
            //if (DatabaseConnection.CurrentConnection == null)
            //{
            //    DatabaseConnection.CurrentConnection = _databaseConfig.GetDatabaseConnection();
            //}
            //BORegistry.DataAccessor = new DataAccessorDB();
            BORegistry.DataAccessor = new DataAccessorInMemory();
        }

        [TestFixtureSetUp]
        public void TestFixtureSetup()
        {
            //Code that is executed before any test is run in this class. If multiple tests
            // are executed then it will still only be called once
            UITestUtils.SetupFixture();
        }

        [TearDown]
        public void TearDownTest()
        {
            //runs every time any testmethod is complete
        }

        [Test]
        public void Test_ConstructCreateGridController()
        {
            //---------------Set up test pack-------------------

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            //---------------Test Result -----------------------
            Assert.IsInstanceOf(typeof (IWizardController), wizardController);
            Assert.AreEqual(2, wizardController.StepCount);
            Assert.IsInstanceOf(typeof (SelectPropertiesWizardStep), wizardController.GetFirstStep());
            Assert.IsInstanceOf(typeof (SelectPropertiesFilterWizardStep), wizardController.GetNextStep());
        }

        //TODO: if no filter props are selected should not create filter def
        //TODO: If no grid props are selected should not create grid def.

        [Test]
        public void Test_SetClass_ShouldSetClassToWizardStep()
        {
            //---------------Set up test pack-------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            SelectPropertiesWizardStep propertiesWizardStep1 =
                (SelectPropertiesWizardStep) wizardController.GetFirstStep();
            SelectPropertiesFilterWizardStep propertiesWizardStep2 =
                (SelectPropertiesFilterWizardStep) wizardController.GetNextStep();
            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            dmClass.Properties.CreateBusinessObject().PropertyName = TestUtilsShared.GetRandomString();
            dmClass.Properties.CreateBusinessObject().PropertyName = TestUtilsShared.GetRandomString();
            //---------------Assert Precondition----------------
            Assert.IsNull(wizardController.Class);
            Assert.IsNull(propertiesWizardStep1.SelectedClass);
            Assert.AreNotSame(propertiesWizardStep1, propertiesWizardStep2);
            //---------------Execute Test ----------------------
            wizardController.Class = dmClass;
            //---------------Test Result -----------------------
            Assert.IsNotNull(wizardController.Class);
            Assert.IsNotNull(propertiesWizardStep1.SelectedClass);
            Assert.IsNotNull(propertiesWizardStep2.SelectedClass);
        }

        [Test]
        public void Test_SetSolution_ShouldSetSolutionToWizardStep()
        {
            CreateGridWizardController wizardController = new CreateGridWizardController();
            SelectPropertiesWizardStep propertiesWizardStep =
                (SelectPropertiesWizardStep) wizardController.GetFirstStep();
            SelectPropertiesFilterWizardStep propertiesWizardStep2 =
                (SelectPropertiesFilterWizardStep) wizardController.GetNextStep();
            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();

            DMClass dmClass = solution.AllClasses[1];
            dmClass.Properties.CreateBusinessObject().PropertyName = TestUtilsShared.GetRandomString();
            dmClass.Properties.CreateBusinessObject().PropertyName = TestUtilsShared.GetRandomString();
            //---------------Assert Precondition----------------
            Assert.IsNull(wizardController.Solution);
            Assert.IsNull(propertiesWizardStep.Solution);
            Assert.AreNotSame(propertiesWizardStep, propertiesWizardStep2);
            //---------------Execute Test ----------------------
            wizardController.Solution = solution;
            //---------------Test Result -----------------------
            Assert.IsNotNull(wizardController.Solution);
            Assert.IsNotNull(propertiesWizardStep.Solution);
            Assert.IsNull(propertiesWizardStep2.Solution);
            Assert.AreSame(solution, wizardController.Solution);
            Assert.AreSame(solution, propertiesWizardStep.Solution);
        }

        [Test]
        public void Test_SelectClassForGridCreator_ShouldSetClassForGridFilterCreator()
        {
            //---------------Set up test pack-------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            SelectPropertiesWizardStep gridPropSelector = (SelectPropertiesWizardStep) wizardController.GetFirstStep();
            SelectPropertiesFilterWizardStep gridFilterPropSelector =
                (SelectPropertiesFilterWizardStep) wizardController.GetNextStep();
            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            wizardController.Solution = solution;
            //---------------Assert Precondition----------------
            Assert.IsNull(gridFilterPropSelector.Solution);
            Assert.IsNull(gridFilterPropSelector.SelectedClass);
            Assert.AreSame(solution, wizardController.Solution);
            Assert.AreSame(solution, gridPropSelector.Solution);
            //---------------Execute Test ----------------------
            DMClass selectedDMClass = solution.AllClasses[0];
            gridPropSelector.ClassSelectorControl.SelectedBusinessObject = selectedDMClass;
            //---------------Test Result -----------------------
            Assert.IsNotNull(gridFilterPropSelector.Solution);
            Assert.AreSame(selectedDMClass, gridFilterPropSelector.SelectedClass);
        }

        [Test]
        public void TestAcceptance_Finish_ShouldCreateUIGridWithSelectedProperties()
        {
            //---------------Set up test pack-------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            SelectPropertiesWizardStep propertyColumnsWizardStep =
                (SelectPropertiesWizardStep) wizardController.GetFirstStep();

            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);
            DMProperty selectedProp = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass, false);
            wizardController.Class = dmClass;
            IMultiSelector<string> propertiesSelectorControl = propertyColumnsWizardStep.PropertiesSelectorControl;
            propertiesSelectorControl.Model.Select(selectedProp.PropertyName);
            selectedProp.ObjectIdentity = null;
            wizardController.GetNextStep();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(wizardController.Class);
            Assert.IsNull(wizardController.UIGrid);
            IList<string> availableOptions = propertiesSelectorControl.Model.AvailableOptions;
            Assert.AreEqual(1, availableOptions.Count);
            IList<string> selectedOptions = propertiesSelectorControl.SelectedOptions;
            Assert.AreEqual(1, selectedOptions.Count);
            Assert.IsFalse(selectedProp.IsPartOfObjectIdentity);
            Assert.IsFalse(selectedProp.KeepValuePrivate.Value);
            //---------------Execute Test ----------------------
            wizardController.Finish();
            //---------------Test Result -----------------------
            UIGridInfo gridInfo = wizardController.UIGrid;
            Assert.IsNotNull(gridInfo);
            BusinessObjectCollection<UIGridColumnInfo> gridColumnInfos = gridInfo.UIColumns;
            Assert.AreEqual(1, gridColumnInfos.Count);
            UIGridColumnInfo gridColumnInfo = gridColumnInfos[0];
            Assert.AreEqual(selectedProp.PropertyName, gridColumnInfo.PropertyName);
        }

        [Test]
        public void
            TestAcceptance_Finish_ShouldCreateUIGridWithSelectedProperties_AndShouldCreateUIGridFilterWithSelectedProperties
            ()
        {
            //---------------Set up test pack-------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            SelectPropertiesWizardStep propertyColumnsWizardStep =
                (SelectPropertiesWizardStep) wizardController.GetFirstStep();


            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            DMProperty property = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);
            property.PropertyName = TestUtilsShared.GetRandomString();
            DMProperty selectedProp = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass, false);
            selectedProp.PropertyName = TestUtilsShared.GetRandomString();
            //wizardController.Class = dmClass;
            wizardController.Solution = solution;
            IMultiSelector<string> propertiesSelectorControl = propertyColumnsWizardStep.PropertiesSelectorControl;
            propertyColumnsWizardStep.ClassSelectorControl.SelectedBusinessObject = dmClass;
            propertiesSelectorControl.Model.Select(selectedProp.PropertyName);
            selectedProp.ObjectIdentity = null;
            SelectPropertiesFilterWizardStep propertyFilterWizardStep =
                (SelectPropertiesFilterWizardStep) wizardController.GetNextStep();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(wizardController.Class);
            Assert.IsNull(wizardController.UIGrid);
            IList<string> availableOptions = propertiesSelectorControl.Model.AvailableOptions;
            Assert.AreEqual(1, availableOptions.Count);
            IList<string> selectedOptions = propertiesSelectorControl.SelectedOptions;
            Assert.AreEqual(1, selectedOptions.Count);
            Assert.IsFalse(selectedProp.IsPartOfObjectIdentity);
            Assert.IsFalse(selectedProp.KeepValuePrivate.Value);
            Assert.IsNull(wizardController.GridFilter);
            Assert.IsFalse(propertyFilterWizardStep.ClassSelectorControl.Visible);
            Assert.AreSame(propertyColumnsWizardStep.SelectedClass, propertyFilterWizardStep.SelectedClass);
            //---------------Execute Test ----------------------
            propertyFilterWizardStep.PropertiesSelectorControl.Model.Select(selectedProp.PropertyName);
            wizardController.Finish();
            //---------------Test Result -----------------------
            UIGridInfo gridInfo = wizardController.UIGrid;
            Assert.IsNotNull(gridInfo);
            BusinessObjectCollection<UIGridColumnInfo> gridColumnInfos = gridInfo.UIColumns;
            Assert.AreEqual(1, gridColumnInfos.Count);
            UIGridColumnInfo gridColumnInfo = gridColumnInfos[0];
            Assert.AreEqual(selectedProp.PropertyName, gridColumnInfo.PropertyName);

            UIGridFilter gridFilter = wizardController.GridFilter;
            Assert.IsNotNull(gridFilter);
            BusinessObjectCollection<UIGridFilterProp> gridFilterProps = gridFilter.UIGridFilterProps;
            Assert.AreEqual(1, gridFilterProps.Count);
            UIGridFilterProp gridFilterProp = gridFilterProps[0];
            Assert.AreEqual(selectedProp.PropertyName, gridFilterProp.PropertyName);
            Assert.AreSame(gridInfo, gridFilter.UIGridInfo);
            Assert.IsNotNull(gridInfo.UIView);
            Assert.AreEqual(1, dmClass.UIViews.Count);
            UIView uiView = dmClass.UIViews[0];
            Assert.AreSame(uiView, gridInfo.UIView);
            Assert.IsNotNull(uiView.UIFormInfo);
            Assert.AreEqual(dmClass.Properties.Count, uiView.UIFormInfo.UIFields.Count);
            Assert.AreEqual("default",uiView.ViewName);
        }

        [Test]
        public void TestAcceptance_Finish_ShouldNameViewUniquely()
        {
            //---------------Set up test pack-------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            SelectPropertiesWizardStep propertyColumnsWizardStep =
                (SelectPropertiesWizardStep)wizardController.GetFirstStep();


            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            UIView uiView1 = dmClass.UIViews.CreateBusinessObject();
            DMProperty property = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);
            property.PropertyName = TestUtilsShared.GetRandomString();
            DMProperty selectedProp = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass, false);
            selectedProp.PropertyName = TestUtilsShared.GetRandomString();
            //wizardController.Class = dmClass;
            wizardController.Solution = solution;
            IMultiSelector<string> propertiesSelectorControl = propertyColumnsWizardStep.PropertiesSelectorControl;
            propertyColumnsWizardStep.ClassSelectorControl.SelectedBusinessObject = dmClass;
            propertiesSelectorControl.Model.Select(selectedProp.PropertyName);
            selectedProp.ObjectIdentity = null;
            SelectPropertiesFilterWizardStep propertyFilterWizardStep =
                (SelectPropertiesFilterWizardStep)wizardController.GetNextStep();
            //---------------Assert Precondition----------------
            Assert.AreEqual(1, dmClass.UIViews.Count);
            Assert.AreEqual("default", uiView1.ViewName);
            Assert.IsNotNull(wizardController.Class);
            Assert.IsNull(wizardController.UIGrid);
            IList<string> availableOptions = propertiesSelectorControl.Model.AvailableOptions;
            Assert.AreEqual(1, availableOptions.Count);
            IList<string> selectedOptions = propertiesSelectorControl.SelectedOptions;
            Assert.AreEqual(1, selectedOptions.Count);
            Assert.IsFalse(selectedProp.IsPartOfObjectIdentity);
            Assert.IsFalse(selectedProp.KeepValuePrivate.Value);
            Assert.IsNull(wizardController.GridFilter);
            Assert.IsFalse(propertyFilterWizardStep.ClassSelectorControl.Visible);
            Assert.AreSame(propertyColumnsWizardStep.SelectedClass, propertyFilterWizardStep.SelectedClass);
            //---------------Execute Test ----------------------
            propertyFilterWizardStep.PropertiesSelectorControl.Model.Select(selectedProp.PropertyName);
            wizardController.Finish();
            //---------------Test Result -----------------------
            UIGridInfo gridInfo = wizardController.UIGrid;
            Assert.IsNotNull(gridInfo);
            BusinessObjectCollection<UIGridColumnInfo> gridColumnInfos = gridInfo.UIColumns;
            Assert.AreEqual(1, gridColumnInfos.Count);
            UIGridColumnInfo gridColumnInfo = gridColumnInfos[0];
            Assert.AreEqual(selectedProp.PropertyName, gridColumnInfo.PropertyName);

            UIGridFilter gridFilter = wizardController.GridFilter;
            Assert.IsNotNull(gridFilter);
            BusinessObjectCollection<UIGridFilterProp> gridFilterProps = gridFilter.UIGridFilterProps;
            Assert.AreEqual(1, gridFilterProps.Count);
            UIGridFilterProp gridFilterProp = gridFilterProps[0];
            Assert.AreEqual(selectedProp.PropertyName, gridFilterProp.PropertyName);
            Assert.AreSame(gridInfo, gridFilter.UIGridInfo);
            Assert.IsNotNull(gridInfo.UIView);
            Assert.AreEqual(2, dmClass.UIViews.Count);
            UIView uiView = dmClass.UIViews[1];
            Assert.AreSame(uiView, gridInfo.UIView);
            Assert.IsNotNull(uiView.UIFormInfo);
            Assert.AreEqual(dmClass.Properties.Count, uiView.UIFormInfo.UIFields.Count);
            Assert.AreEqual("default1", uiView.ViewName);
        }

        [Test]
        public void TestAcceptance_Finish_ShouldCreateUIGridAndUIGridFilterWithSelectedProperties()
        {
            //---------------Set up test pack-------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            IWizardControl wzrdControl = GlobalUIRegistry.ControlFactory.CreateWizardControl(wizardController);
            wzrdControl.Start();
            SelectPropertiesWizardStep gridPropSelector = (SelectPropertiesWizardStep) wizardController.GetFirstStep();

            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);
            DMProperty selectedProp = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);

            wizardController.Class = dmClass;
            IMultiSelector<string> propertiesMultiSelectorControl = gridPropSelector.PropertiesSelectorControl;
            propertiesMultiSelectorControl.Model.Select(selectedProp.PropertyName);
            selectedProp.ObjectIdentity = null;
            selectedProp.KeepValuePrivate = false;
            //---------------Assert Precondition----------------
            Assert.AreSame(dmClass, wizardController.Class);
            Assert.AreEqual(0, dmClass.UIViews.Count);
            Assert.IsNull(wizardController.GridFilter);
            IList<string> availableOptions = propertiesMultiSelectorControl.Model.AvailableOptions;
            Assert.AreEqual(1, availableOptions.Count);
            IList<string> selectedOptions = propertiesMultiSelectorControl.SelectedOptions;
            Assert.AreEqual(1, selectedOptions.Count);
            Assert.IsFalse(selectedProp.IsPartOfObjectIdentity);
            Assert.IsFalse(selectedProp.KeepValuePrivate.Value);
            SelectPropertiesFilterWizardStep gridFilterPropSelector = wizardController.SelectFilterPropertiesWizardStep;
            Assert.AreEqual
                (dmClass.Properties.Count, gridFilterPropSelector.PropertiesSelectorControl.Model.AvailableOptions.Count,
                 "Should have all the Classes Properties available");
            Assert.IsTrue(wizardController.IsFirstStep());
            //---------------Execute First Step ----------------------
            wzrdControl.NextButton.PerformClick();
            //---------------Assert Results First Step ---------------
            Assert.IsFalse(wizardController.IsFirstStep());
            //Should Set the Available Options in the Grid Filter to the columns
            // selected in the Grid.
            Assert.AreEqual(1, gridFilterPropSelector.PropertiesSelectorControl.Model.AvailableOptions.Count);

            //--------------Select the Property ----------------------
            IMultiSelector<string> filterPropertiesMultiSelectorControl =
                gridFilterPropSelector.PropertiesSelectorControl;
            filterPropertiesMultiSelectorControl.Model.Select(selectedProp.PropertyName);

            //--------------Assert Preconditions to Finishd-------------
            IList<string> selectedFilterProps = propertiesMultiSelectorControl.SelectedOptions;
            Assert.AreEqual(1, selectedFilterProps.Count);
            Assert.IsNull(wizardController.UIGrid);
            Assert.IsNull(wizardController.GridFilter);

            //--- Execute Second Step (Finish Wizard) ----------------
            wzrdControl.NextButton.PerformClick();
            //----Assert Results Second Step--------------------------
            //Should Create Grid with selected property
            UIGridInfo grid = wizardController.UIGrid;
            Assert.IsNotNull(grid);
            Assert.IsNotNull(grid.UIGridFilter);
            Assert.AreEqual(1, grid.UIColumns.Count);
            //Should Create Grid Filter with Selected Properties.
            Assert.IsNotNull(wizardController.GridFilter);
            UIGridFilter gridFilter = wizardController.GridFilter;
            Assert.AreSame(grid.UIGridFilter, gridFilter);
            Assert.AreEqual(1, gridFilter.UIGridFilterProps.Count);

            //-----------------Verify That A UIView is created for the Class with
            //  The appropriate Grid and Form Infos
            Assert.AreEqual(1, dmClass.UIViews.Count);
            UIView uiView = dmClass.UIViews[0];
            Assert.IsNotNull(uiView.UIGridInfo);
            Assert.AreSame(grid, uiView.UIGridInfo);
            Assert.IsNotNull(uiView.UIFormInfo);
            Assert.AreEqual(2, uiView.UIFormInfo.UIFields.Count);
            Assert.IsFalse(uiView.Status.IsNew);
        }


        [Test]
        public void TestAcceptance_Finish_ShouldCreateUIGridWithSelectedProperties_NoFilterDefIfNoSelectedProperties()
        {
            //---------------Set up test pack-------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            IWizardControl wzrdControl = GlobalUIRegistry.ControlFactory.CreateWizardControl(wizardController);
            wzrdControl.Start();
            SelectPropertiesWizardStep gridPropSelector = (SelectPropertiesWizardStep) wizardController.GetFirstStep();

            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);
            DMProperty selectedProp = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);

            wizardController.Class = dmClass;
            IMultiSelector<string> propertiesMultiSelectorControl = gridPropSelector.PropertiesSelectorControl;
            propertiesMultiSelectorControl.Model.Select(selectedProp.PropertyName);
            selectedProp.ObjectIdentity = null;
            selectedProp.KeepValuePrivate = false;
            //---------------Assert Precondition----------------
            Assert.AreSame(dmClass, wizardController.Class);
            Assert.AreEqual(0, dmClass.UIViews.Count);
            Assert.IsNull(wizardController.GridFilter);
            IList<string> availableOptions = propertiesMultiSelectorControl.Model.AvailableOptions;
            Assert.AreEqual(1, availableOptions.Count);
            IList<string> selectedOptions = propertiesMultiSelectorControl.SelectedOptions;
            Assert.AreEqual(1, selectedOptions.Count);
            Assert.IsFalse(selectedProp.IsPartOfObjectIdentity);
            Assert.IsFalse(selectedProp.KeepValuePrivate.Value);
            SelectPropertiesFilterWizardStep gridFilterPropSelector = wizardController.SelectFilterPropertiesWizardStep;
            Assert.AreEqual
                (dmClass.Properties.Count, gridFilterPropSelector.PropertiesSelectorControl.Model.AvailableOptions.Count,
                 "Should have all the Classes Properties available");
            Assert.IsTrue(wizardController.IsFirstStep());
            //---------------Execute First Step ----------------------
            wzrdControl.NextButton.PerformClick();
            //---------------Assert Results First Step ---------------
            Assert.IsFalse(wizardController.IsFirstStep());
            //Should Set the Available Options in the Grid Filter to the columns
            // selected in the Grid.
            Assert.AreEqual(1, gridFilterPropSelector.PropertiesSelectorControl.Model.AvailableOptions.Count);

            //--------------Select the Property ----------------------
            IMultiSelector<string> filterPropertiesMultiSelectorControl =
                gridFilterPropSelector.PropertiesSelectorControl;
            filterPropertiesMultiSelectorControl.SelectedOptionsListBox.ClearSelected();
            //--------------Assert Preconditions to Finish-------------
            IList<string> selectedFilterProps = filterPropertiesMultiSelectorControl.SelectedOptions;
            Assert.AreEqual(0, selectedFilterProps.Count);
            Assert.IsNull(wizardController.UIGrid);
            Assert.IsNull(wizardController.GridFilter);

            //--- Execute Second Step (Finish Wizard) ----------------
            wzrdControl.NextButton.PerformClick();
            //----Assert Results Second Step--------------------------
            //Should Create Grid with selected property
            UIGridInfo grid = wizardController.UIGrid;
            Assert.IsNotNull(grid);
            Assert.AreEqual(1, grid.UIColumns.Count);
            //Should Not Create Grid Filter if no Selected Properties.
            Assert.IsNull(wizardController.GridFilter);
            Assert.IsNull(grid.UIGridFilter);

            //-----------------Verify That A UIView is created for the Class with
            //  The appropriate Grid and Form Infos
            Assert.AreEqual(1, dmClass.UIViews.Count);
            UIView uiView = dmClass.UIViews[0];
            Assert.IsNotNull(uiView.UIGridInfo);
            Assert.AreSame(grid, uiView.UIGridInfo);
            Assert.IsNotNull(uiView.UIFormInfo);
            Assert.AreEqual(2, uiView.UIFormInfo.UIFields.Count);
        }


        [Test]
        public void Test_CannotMoveToNextStep_IfNoSelectedColumnProperties()
        {
            //--------------- Set up test pack ------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            SelectPropertiesWizardStep columnPropertiesWizardStep =
                (SelectPropertiesWizardStep) wizardController.GetFirstStep();
            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            dmClass.Properties.CreateBusinessObject().PropertyName = TestUtilsShared.GetRandomString();
            dmClass.Properties.CreateBusinessObject().PropertyName = TestUtilsShared.GetRandomString();
            wizardController.Solution = solution;
            wizardController.Class = dmClass;
            //--------------- Test Preconditions ----------------
            Assert.IsNotNull(wizardController.Class);
            Assert.IsNotNull(columnPropertiesWizardStep.SelectedClass);
            Assert.AreEqual(0, columnPropertiesWizardStep.PropertiesSelectorControl.Model.SelectedOptions.Count);
            //--------------- Execute Test ----------------------
            string message;
            bool canMoveOn = wizardController.CanMoveOn(out message);
            //--------------- Test Result -----------------------
            Assert.IsFalse(canMoveOn);
            StringAssert.Contains("You must select properties to show in the grid before you can move on", message);
        }

        [Test]
        public void Test_CannotMoveToNextStep_IfSelectedClassHasUIViews()
        {
            //--------------- Set up test pack ------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            SelectPropertiesWizardStep columnPropertiesWizardStep =
                (SelectPropertiesWizardStep) wizardController.GetFirstStep();
            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            DMProperty dmProperty = dmClass.Properties.CreateBusinessObject();
            dmProperty.PropertyName = TestUtilsShared.GetRandomString();
            dmClass.Properties.CreateBusinessObject().PropertyName = TestUtilsShared.GetRandomString();
            wizardController.Solution = solution;
            wizardController.Class = dmClass;
            dmClass.UIViews.CreateBusinessObject();
            columnPropertiesWizardStep.PropertiesSelectorControl.Model.Select(dmProperty.PropertyName);
            //--------------- Test Preconditions ----------------
            Assert.IsNotNull(wizardController.Class);
            Assert.IsNotNull(columnPropertiesWizardStep.SelectedClass);
            Assert.AreEqual(1, columnPropertiesWizardStep.PropertiesSelectorControl.Model.SelectedOptions.Count);
            Assert.AreEqual(1, dmClass.UIViews.Count);
            //--------------- Execute Test ----------------------
            string message;
            try
            {
                wizardController.CanMoveOn(out message);
            }
                //--------------- Test Result -----------------------
            catch (UserException ex)
            {
                StringAssert.Contains("You cannot create a grid definition for the entity '" + dmClass.ToString()
                                      + "' as it already has a UIView defined.", ex.Message);
            }
        }

        [Test]
        public void Test_FilterStep2_ShouldOnlyShowSelectedOptionsFromStep1()
        {
            //--------------- Set up test pack ------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            SelectPropertiesWizardStep propertyColumnsWizardStep =
                (SelectPropertiesWizardStep) wizardController.GetFirstStep();

            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);
            DMProperty selectedProp = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass, false);
            wizardController.Class = dmClass;
            IMultiSelector<string> propertiesSelectorControl = propertyColumnsWizardStep.PropertiesSelectorControl;
            propertiesSelectorControl.Model.Select(selectedProp.PropertyName);
            SelectPropertiesFilterWizardStep propertyColumnsWizardStep2 =
                (SelectPropertiesFilterWizardStep) wizardController.GetNextStep();
            wizardController.GetPreviousStep();
            IMultiSelector<string> propertiesSelectorControl2 = propertyColumnsWizardStep2.PropertiesSelectorControl;
            //---------------Assert Precondition----------------
            Assert.IsNotNull(wizardController.Class);
            Assert.IsNull(wizardController.UIGrid);
            IList<string> availableOptions = propertiesSelectorControl.Model.AvailableOptions;
            Assert.AreEqual(1, availableOptions.Count);
            IList<string> selectedOptions = propertiesSelectorControl.SelectedOptions;
            Assert.AreEqual(1, selectedOptions.Count);
            Assert.IsFalse(selectedProp.IsPartOfObjectIdentity);
            Assert.IsFalse(selectedProp.KeepValuePrivate.Value);
            IList<string> availableFilterOptions = propertiesSelectorControl2.Model.AvailableOptions;
            Assert.AreEqual(2, availableFilterOptions.Count);

            //--------------- Execute Test ----------------------
            string message;
            bool canMoveOn = wizardController.CanMoveOn(out message);

            //--------------- Test Result -----------------------
            Assert.IsTrue(canMoveOn);
            Assert.AreEqual("", message);
            availableFilterOptions = propertiesSelectorControl2.Model.AvailableOptions;
            Assert.AreEqual(1, availableFilterOptions.Count);
            Assert.IsTrue(availableFilterOptions.Contains(selectedProp.PropertyName));
        }

        //[Test]
        //public void Test_SetClass_ShouldRaiseError_WhenSelectedClassAlreadyHasUIView()
        //{
        //    //--------------- Set up test pack ------------------
        //    CreateGridWizardController wizardController = new CreateGridWizardController();

        //    DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
        //    DMClass dmClass = solution.AllClasses[1];
        //    dmClass.UIViews.CreateBusinessObject();
        //    TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);
        //    //--------------- Test Preconditions ----------------
        //    Assert.AreEqual(1, dmClass.UIViews.Count);
        //    //--------------- Execute Test ----------------------
        //    try
        //    {
        //        wizardController.Class = dmClass;
        //        Assert.Fail("expected Err");
        //    }
        //    //---------------Test Result -----------------------
        //    catch (UserException ex)
        //    {
        //        StringAssert.Contains("You cannot create a grid definition for the entity '" + dmClass.ToString()
        //            + "' as it already has a UIView defined.", ex.Message);
        //    }
        //}

        [Test]
        public void Test_SetClass_ShouldNotRaiseError_WhenClassIsNull()
        {
            //--------------- Set up test pack ------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();

            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            wizardController.Class = dmClass;
            //--------------- Test Preconditions ----------------
            Assert.IsNotNull(wizardController.Class);
            //--------------- Execute Test ----------------------
            wizardController.Class = null;

            //---------------Test Result -----------------------
            Assert.IsNull(wizardController.Class);
        }

        [Test]
        public void TestAcceptance_FixBug_DeleteDMClassAndPropThatHasUIViewCreated_ShouldNotGiveError()
        {
            //---------------Set up test pack-------------------
            //   BORegistry.DataAccessor = new DataAccessorDB();
            CreateGridWizardController wizardController = new CreateGridWizardController();
            SelectPropertiesWizardStep propertyColumnsWizardStep =
                (SelectPropertiesWizardStep) wizardController.GetFirstStep();

            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);
            DMProperty selectedProp = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass, false);
            wizardController.Class = dmClass;
            IMultiSelector<string> propertiesSelectorControl = propertyColumnsWizardStep.PropertiesSelectorControl;
            propertiesSelectorControl.Model.Select(selectedProp.PropertyName);
            selectedProp.ObjectIdentity = null;
            wizardController.GetNextStep();
            wizardController.Finish();
            dmClass.Save();
            //---------------Assert Precondition----------------
            UIGridInfo gridInfo = wizardController.UIGrid;
            Assert.IsNotNull(gridInfo);
            BusinessObjectCollection<UIGridColumnInfo> gridColumnInfos = gridInfo.UIColumns;
            Assert.AreEqual(1, gridColumnInfos.Count);
            UIGridColumnInfo gridColumnInfo = gridColumnInfos[0];
            Assert.AreEqual(selectedProp.PropertyName, gridColumnInfo.PropertyName);
            //---------------Execute Test ----------------------
            dmClass.MarkForDelete();
            dmClass.Save();
            //---------------Test Result -----------------------
            Assert.IsTrue(dmClass.Status.IsDeleted);
            Assert.IsTrue(selectedProp.Status.IsDeleted);
            Assert.IsTrue(gridColumnInfo.Status.IsDeleted);
        }

        [Test]
        public void TestAcceptance_Finish_ShouldCreateView_WithUniqueName()
        {
            //---------------Set up test pack-------------------
            CreateGridWizardController wizardController = new CreateGridWizardController();
            IWizardControl wzrdControl = GlobalUIRegistry.ControlFactory.CreateWizardControl(wizardController);
            wzrdControl.Start();
            SelectPropertiesWizardStep gridPropSelector = (SelectPropertiesWizardStep)wizardController.GetFirstStep();

            DMSolution solution = TestUtilsDMSolution.CreateSavedDMSolutionWithTwoClasses();
            DMClass dmClass = solution.AllClasses[1];
            UIView uiView1 = dmClass.UIViews.CreateBusinessObject();
            TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);
            DMProperty selectedProp = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass);

            wizardController.Class = dmClass;
            IMultiSelector<string> propertiesMultiSelectorControl = gridPropSelector.PropertiesSelectorControl;
            propertiesMultiSelectorControl.Model.Select(selectedProp.PropertyName);
            selectedProp.ObjectIdentity = null;
            selectedProp.KeepValuePrivate = false;
            //---------------Assert Precondition----------------
            Assert.AreSame(dmClass, wizardController.Class);
            Assert.AreEqual(1, dmClass.UIViews.Count);
            Assert.AreEqual("default", dmClass.UIViews[0].ViewName);
            Assert.IsNull(wizardController.GridFilter);
            IList<string> availableOptions = propertiesMultiSelectorControl.Model.AvailableOptions;
            Assert.AreEqual(1, availableOptions.Count);
            IList<string> selectedOptions = propertiesMultiSelectorControl.SelectedOptions;
            Assert.AreEqual(1, selectedOptions.Count);
            Assert.IsFalse(selectedProp.IsPartOfObjectIdentity);
            Assert.IsFalse(selectedProp.KeepValuePrivate.Value);
            SelectPropertiesFilterWizardStep gridFilterPropSelector = wizardController.SelectFilterPropertiesWizardStep;
            Assert.AreEqual
                (dmClass.Properties.Count, gridFilterPropSelector.PropertiesSelectorControl.Model.AvailableOptions.Count,
                 "Should have all the Classes Properties available");
            Assert.IsTrue(wizardController.IsFirstStep());
            //---------------Execute First Step ----------------------
            wzrdControl.NextButton.PerformClick();
            //---------------Assert Results First Step ---------------
            Assert.IsFalse(wizardController.IsFirstStep());
            //Should Set the Available Options in the Grid Filter to the columns
            // selected in the Grid.
            Assert.AreEqual(1, gridFilterPropSelector.PropertiesSelectorControl.Model.AvailableOptions.Count);

            //--------------Select the Property ----------------------
            IMultiSelector<string> filterPropertiesMultiSelectorControl =
                gridFilterPropSelector.PropertiesSelectorControl;
            filterPropertiesMultiSelectorControl.SelectedOptionsListBox.ClearSelected();
            //--------------Assert Preconditions to Finish-------------
            IList<string> selectedFilterProps = filterPropertiesMultiSelectorControl.SelectedOptions;
            Assert.AreEqual(0, selectedFilterProps.Count);
            Assert.IsNull(wizardController.UIGrid);
            Assert.IsNull(wizardController.GridFilter);

            //--- Execute Second Step (Finish Wizard) ----------------
            wzrdControl.NextButton.PerformClick();
            //----Assert Results Second Step--------------------------
            //Should Create Grid with selected property
            UIGridInfo grid = wizardController.UIGrid;
            Assert.IsNotNull(grid);
            Assert.AreEqual(1, grid.UIColumns.Count);
            //Should Not Create Grid Filter if no Selected Properties.
            Assert.IsNull(wizardController.GridFilter);
            Assert.IsNull(grid.UIGridFilter);

            //-----------------Verify That A UIView is created for the Class with
            //  The appropriate Grid and Form Infos
            Assert.AreEqual(2, dmClass.UIViews.Count);
            UIView uiView = dmClass.UIViews[1];
            Assert.AreEqual("default1", uiView.ViewName);
            Assert.IsNotNull(uiView.UIGridInfo);
            Assert.AreSame(grid, uiView.UIGridInfo);
            Assert.IsNotNull(uiView.UIFormInfo);
            Assert.AreEqual(2, uiView.UIFormInfo.UIFields.Count);
        }
    }
}