using System;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using Habanero.BO;
using Habanero.Faces.Base;
using Habanero.Faces.Win;

namespace FireStarterModeller.UI
{
    public class DatabaseConnectionInfoControl : UserControlWin
    {
        private readonly IButton _testConnectionButton;
        private readonly IBOGridAndEditorControl _boGridAndEditorControl;
        private readonly IControlFactory _controlFactory;

        public DatabaseConnectionInfoControl()
        {
            _controlFactory = GlobalUIRegistry.ControlFactory;

            _boGridAndEditorControl = _controlFactory.CreateGridAndBOEditorControl<DatabaseConnectionInfo>();

            BOEditorControlWin<DatabaseConnectionInfo> boEditorControl =  (BOEditorControlWin<DatabaseConnectionInfo>) _boGridAndEditorControl.IBOEditorControl;
            IPanelInfo panelInfo = boEditorControl.PanelInfo;
            IPanel panel = panelInfo.Panel;
            IButtonGroupControl testConnectionButtonGroupControl = _controlFactory.CreateButtonGroupControl();
            _testConnectionButton = testConnectionButtonGroupControl.AddButton("Test Connection", TestConnection_ClickHandler);
            testConnectionButtonGroupControl.Dock = DockStyle.Bottom;
            panel.Controls.Add(testConnectionButtonGroupControl);

            BorderLayoutManager layoutManager = _controlFactory.CreateBorderLayoutManager(this);
            layoutManager.AddControl(_boGridAndEditorControl, BorderLayoutManager.Position.Centre);

            BusinessObjectCollection<DatabaseConnectionInfo> databaseConnectionInfos = BORegistry.DataAccessor.BusinessObjectLoader.GetBusinessObjectCollection<DatabaseConnectionInfo>("", "Database");
            _boGridAndEditorControl.BusinessObjectCollection = databaseConnectionInfos;
        }

        public IBOGridAndEditorControl BOGridAndEditorControl
        {
            get { return _boGridAndEditorControl; }
        }

        private IControlFactory ControlFactory
        {
            get { return _controlFactory; }
        }

        private void TestConnection_ClickHandler(object sender, EventArgs args)
        {
            DatabaseConnectionInfo databaseConnectionInfo = (DatabaseConnectionInfo)_boGridAndEditorControl.IBOEditorControl.BusinessObject;
            if (databaseConnectionInfo != null)
            {
                string message;
                bool isDatabaseConnectionValid = databaseConnectionInfo.IsDatabaseConnectionValid(out message);
                if (isDatabaseConnectionValid)
                {
                    this.ControlFactory.ShowMessageBox("Connection tested OK");
                }
                else
                {
                    this.ControlFactory.ShowMessageBox(message);
                }
            }
        }

        public IButton TestConnectionButton
        {
            get { return _testConnectionButton; }
        }
    }
}