using Habanero.UI.Base;
using Habanero.UI.Win;

namespace FireStarterModeller.UI
{
    ///<summary>
    /// A <see cref="ITextBox"/> with a <see cref="Button"/> next to it on the right with a '...' displayed as the text.
    ///</summary>
    public class ExtendedTextBoxWin : UserControlWin
    {
        private readonly IControlFactory _controlFactory;
        private readonly ITextBox _textBox;
        private readonly IButton _button;

        ///<summary>
        /// Constructs the <see cref="ExtendedTextBoxWin"/> with the default <see cref="IControlFactory"/>.
        ///</summary>
        public ExtendedTextBoxWin() : this(GlobalUIRegistry.ControlFactory) { }

        ///<summary>
        /// Constructs the <see cref="ExtendedTextBoxWin"/> with the specified <see cref="IControlFactory"/>.
        ///</summary>
        public ExtendedTextBoxWin(IControlFactory controlFactory)
        {
            _controlFactory = controlFactory;
            IUserControlHabanero userControlHabanero = this;
            _textBox = _controlFactory.CreateTextBox();
            _button = _controlFactory.CreateButton("...");
            BorderLayoutManager borderLayoutManager = controlFactory.CreateBorderLayoutManager(userControlHabanero);
            borderLayoutManager.AddControl(_textBox, BorderLayoutManager.Position.Centre);
            borderLayoutManager.AddControl(_button, BorderLayoutManager.Position.East);
        }


        ///<summary>
        /// Returns the <see cref="ITextBox"/> in the control
        ///</summary>
        public ITextBox TextBox
        {
            get { return _textBox; }
        }

        ///<summary>
        /// Returns the <see cref="IButton"/> in the control
        ///</summary>
        public IButton Button
        {
            get { return _button; }
        }
    }
}