using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Data.OleDb;

namespace Firestarter.DB.Schema.Logic
{
    public class  DBSchemaExtractor_Access : DBSchemaExtractor
    {
        [Flags]
        public enum OleDbColumnFlags
        {
            ISBOOKMARK = 1,
            MAYDEFER = 2,
            WRITE = 4,
            WRITEUNKNOWN = 8,
            ISFIXEDLENGTH = 16,
            ISNULLABLE = 32,
            MAYBENULL = 64,
            ISLONG = 128,
            ISROWID = 256,
            ISROWVER = 512,
            CACHEDEFERRED = 1024,
            SCALEISNEGATIVE = 2048,
            RESERVED = 4096,
            ISROWURL = 8192,
            ISDEFAULTSTREAM = 16384,
            ISCOLLECTION = 32768,
            ISSTREAM = 65536,
            ISROWSET = 131072,
            ISROW = 262144,
            ROWSPECIFICCOLUMN = 524288
        }


        private List<string> _systemTables;

        public DBSchemaExtractor_Access(IDbConnection idbConnection) : base(idbConnection)
        {
            AddSystemTables(GetTablesDataTable(null, null, "SYSTEM TABLE"));
            AddSystemTables(GetTablesDataTable(null, null, "ACCESS TABLE"));
        }

        private void AddSystemTables(DataTable dataTable)
        {
            _systemTables = new List<string>();
            foreach (DataRow row in dataTable.Rows)
            {
                string tableName = row[DBSchemaTemplate.Naming.TableName].ToString();
                _systemTables.Add(tableName);
            }
        }

        public override string CatalogName { get { return null; } }
        public override string SchemaName { get { return null; } }

        protected override DbDataAdapter CreateDataAdapter(string sql) 
        {
            return new OleDbDataAdapter(sql, (OleDbConnection) _dbConnection);
        }

        public override DataTable GetDatabasesDataTable(string catalogName, string schemaName)
        {
            DBSchemaTemplate dbSchemaTemplate = new DBSchemaTemplate();
            DataTable dataTable = dbSchemaTemplate.CreateDatabasesDataTable();
            dataTable.Rows.Add(new object[] { null, null });
            return dataTable;
        }

        /// <summary>
        /// This method is the specific for the database implementation for fetching the 
        /// DataTable representing the databases for the connection with a specified database name.
        /// This database name might be used to filter the catalog, schema, or both.
        /// </summary>
        /// <param name="databaseName">The name of the database to filter by.</param>
        /// <returns>A DataTable populated with the Databases information.</returns>
        public override DataTable GetDatabasesDataTable(string databaseName)
        {
            return GetDatabasesDataTable(null, null);
        }

        protected override DataTable GetTablesDataTable(string catalogName, string schemaName)
        {
            DataTable dataTable = GetTablesDataTable(catalogName, schemaName, "TABLE");
            dataTable.Columns[DBSchemaTemplate.Naming.TableType].Expression = "'BASE TABLE'";
            return dataTable; 
        }

        private DataTable GetTablesDataTable(string catalogName, string schemaName, string tableType)
        {
            DataTable dataTable = base.GetTablesDataTable(catalogName, schemaName);
            dataTable = GetFilteredDataTable(dataTable, DBSchemaTemplate.Naming.TableType + " = '" + tableType + "'");
            return dataTable;
        }

        private static DataTable GetFilteredDataTable(DataTable dataTable, string filter)
        {
            DataView dataView = dataTable.DefaultView;
            dataView.RowFilter = filter;
            dataTable = dataView.ToTable();
            return dataTable;
        }

        protected override DataTable GetColumnsDataTable(string catalogName, string schemaName, string tableName)
        {
            DataTable dataTable = GetSchemaTable(OleDbSchemaGuid.Columns,
                                             new object[] { catalogName, schemaName, tableName });
            //DataTable dataTable = base.GetColumnsDataTable(catalogName, schemaName, tableName);
            
            dataTable = RemoveSystemTables(dataTable);
            const string oleDbTypeColumnName = "OleDBTypeValue";
            dataTable.Columns[DBSchemaTemplate.Naming.DataType].ColumnName = oleDbTypeColumnName;
            dataTable.Columns.Add(DBSchemaTemplate.Naming.DataType);
            dataTable.Columns.Add(DBSchemaTemplate.Naming.ColumnIsAutoIncrement);
            foreach (DataRow dataRow in dataTable.Rows)
            {
                object dataTypeValue = dataRow[oleDbTypeColumnName];
                long columnFlagsValue = (long)dataRow["COLUMN_FLAGS"];
                OleDbColumnFlags columnFlags = (OleDbColumnFlags) columnFlagsValue;
                OleDbType oleDbType = (OleDbType)dataTypeValue;
                bool isAutoIncrement = false;
                //if (oleDbType == OleDbType.Integer && columnFlagsValue == 122)
                //{
                //    isAutoIncrement = true; //TODO Mark 20 Apr 2009: Need to implement this somehow (it seems impossible).
                //}
                dataRow[DBSchemaTemplate.Naming.ColumnIsAutoIncrement] = isAutoIncrement.ToString();
                string dataType = oleDbType.ToString();
                if (oleDbType == OleDbType.WChar)
                {
                    if ((columnFlags & OleDbColumnFlags.ISLONG) == OleDbColumnFlags.ISLONG)
                    {
                        dataType = "Memo";
                    } else if ((columnFlags & OleDbColumnFlags.ISFIXEDLENGTH) == OleDbColumnFlags.ISFIXEDLENGTH)
                    {
                        dataType = "char";
                    }
                }
                
                dataRow[DBSchemaTemplate.Naming.DataType] = dataType;
            }
            return dataTable;
        }

        private DataTable RemoveSystemTables(DataTable dataTable)
        {
            DataTable newDataTable = dataTable.Clone();
            foreach (DataRow row in dataTable.Rows)
            {
                string currentTableName = row[DBSchemaTemplate.Naming.TableName].ToString();
                if (!_systemTables.Contains(currentTableName))
                {
                    newDataTable.ImportRow(row);
                }
            }
            return newDataTable;
        }

        protected override DataTable GetPrimaryKeyColumnsDataTable(string catalogName, string schemaName, string tableName)
        {
            //DataTable dataTable = GetSchemaDataTable("Indexes", catalogName, null, null, schemaName, tableName);
            //FixIndexColumns(dataTable, "INDEX");
            //dataTable = GetFilteredDataTable(dataTable, "PRIMARY_KEY = true");

            //DataTable dataTable = GetSchemaTable(OleDbSchemaGuid.Primary_Keys,
            //                                     new object[] {catalogName, schemaName, tableName});
            //dataTable.Columns[DBSchemaTemplate.Naming.TableCatalog].ColumnName = DBSchemaTemplate.Naming.KeyCatalog;
            //dataTable.Columns[DBSchemaTemplate.Naming.TableSchema].ColumnName = DBSchemaTemplate.Naming.KeySchema;
            //dataTable.Columns["PK_NAME"].ColumnName = DBSchemaTemplate.Naming.KeyName;
            //dataTable.Columns["ORDINAL"].ColumnName = DBSchemaTemplate.Naming.OrdinalPosition;
            //DataTableUtilities.AddColumn(dataTable, DBSchemaTemplate.Naming.TableCatalog, DBSchemaTemplate.Naming.KeyCatalog);
            //DataTableUtilities.AddColumn(dataTable, DBSchemaTemplate.Naming.TableSchema, DBSchemaTemplate.Naming.KeySchema);

            DataTable dataTable = GetSchemaTable(OleDbSchemaGuid.Indexes,
                                                 new object[] { catalogName, schemaName, null, null, tableName });
            FixIndexColumns(dataTable, "INDEX");
            dataTable = GetFilteredDataTable(dataTable, "PRIMARY_KEY = true");

            dataTable = RemoveSystemTables(dataTable);
            dataTable = GetSortedDataTable(dataTable, String.Format("{0}, {1}, {2}, {3}", 
                DBSchemaTemplate.Naming.KeyCatalog, DBSchemaTemplate.Naming.KeySchema, DBSchemaTemplate.Naming.KeyName, DBSchemaTemplate.Naming.OrdinalPosition ));
            return dataTable;
        }

        private DataTable GetSortedDataTable(DataTable dataTable, string sortClause)
        {
            DataView dataView = dataTable.DefaultView;
            dataView.Sort = sortClause;
            dataTable = dataView.ToTable();
            return dataTable;
        }

        protected override DataTable GetUniqueKeyColumnsDataTable(string catalogName, string schemaName, string tableName)
        {
            DataTable dataTable = GetSchemaDataTable("Indexes", catalogName, schemaName, null, null, tableName);
            FixIndexColumns(dataTable, "INDEX");
            dataTable = GetFilteredDataTable(dataTable, "PRIMARY_KEY = false and UNIQUE = true");
            dataTable = RemoveSystemTables(dataTable);
            return dataTable;
        }

        /// <summary>
        /// Creates a <see cref="DataTable"/> for Foreign Key columns with the following <see cref="DataColumn"/>s:
        /// TABLE_CATALOG, TABLE_SCHEMA, TABLE_NAME, 
        /// KEY_CATALOG, KEY_SCHEMA, KEY_NAME, COLUMN_NAME, REFERENCED_TABLE_CATALOG, 
        /// REFERENCED_TABLE_SCHEMA, REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME, ORDINAL_POSITION
        /// </summary>
        /// <param name="catalogName">The catalog name of the database that you are reading the foreign keys for </param>
        /// <param name="schemaName">The schema name of the database that you are reading the foreign keys for </param>
        /// <param name="tableName">The table name of the particular table that you are reading the foreign keys for </param>
        /// <param name="foreignKeyName">The particular foreign key of the particular table that you are reading the foreign keys for </param>
        /// <returns></returns>
        protected override DataTable GetForeignKeyColumnsDataTable(string catalogName, string schemaName, string tableName, string foreignKeyName)
        {
            DataTable dataTable = GetSchemaTable(OleDbSchemaGuid.Foreign_Keys,
                                             new object[] { null, null, null, catalogName, schemaName, tableName });
            dataTable.Columns["FK_NAME"].ColumnName = DBSchemaTemplate.Naming.KeyName;
            foreach (DataColumn column in dataTable.Columns)
            {
                string columnName = column.ColumnName;
                if (columnName.StartsWith("FK"))
                {
                    column.ColumnName = columnName.Substring(3);
                }
                else if (columnName.StartsWith("PK"))
                {
                    column.ColumnName = "REFERENCED_" + columnName.Substring(3);
                }
                else if (columnName == "ORDINAL")
                {
                    column.ColumnName = DBSchemaTemplate.Naming.OrdinalPosition;
                }
            }
            DataTableUtilities.AddColumn(dataTable, DBSchemaTemplate.Naming.KeyCatalog, DBSchemaTemplate.Naming.TableCatalog);
            DataTableUtilities.AddColumn(dataTable, DBSchemaTemplate.Naming.KeySchema, DBSchemaTemplate.Naming.TableSchema);
            //foreach (DataRow dataRow in schemaTable.Rows)
            //{
            //    string columnName = GetObjectValue<string>(dataRow["FK_COLUMN_NAME"], null);
            //    string refSchemaName = GetObjectValue<string>(dataRow["PK_TABLE_SCHEMA"], null);
            //    string refTableName = GetObjectValue<string>(dataRow["PK_TABLE_NAME"], null);
            //    string refColumnName = GetObjectValue<string>(dataRow["PK_COLUMN_NAME"], null);
            //    string constraintName = GetObjectValue<string>(dataRow["FK_NAME"], null);
            //    //also... "DELETE_RULE" "UPDATE_RULE"
            //    AddRelationshipToTable(tableInfo, columnName, refSchemaName, refTableName, refColumnName, constraintName);
            //}
            dataTable = RemoveSystemTables(dataTable);
            return dataTable;
        }

        private static void FixIndexColumns(DataTable dataTable, string keyNoun)
        {
            if (dataTable == null) return;
            dataTable.Columns[keyNoun + "_CATALOG"].ColumnName = DBSchemaTemplate.Naming.KeyCatalog;
            dataTable.Columns[keyNoun + "_SCHEMA"].ColumnName = DBSchemaTemplate.Naming.KeySchema;
            dataTable.Columns[keyNoun + "_NAME"].ColumnName = DBSchemaTemplate.Naming.KeyName;
        }


        private DataTable GetSchemaTable(Guid schema, object[] restrictions)
        {
            OpenConnection();
            object[] newRestrictions = new object[restrictions.Length];
            for (int count = 0; count < restrictions.Length; count++)
            {
                object value = restrictions[count];
                if (value is string && String.IsNullOrEmpty(value.ToString()))
                {
                    value = null;
                }
                newRestrictions[count] = value;
            }
            return ((OleDbConnection)_dbConnection).GetOleDbSchemaTable(schema, newRestrictions);
        }
    }

//    public class DBSchemaExtractor_Access : DBSchemaExtractor
//    {
//        public DBSchemaExtractor_Access(IDbConnection idbConnection)
//            : base(idbConnection)
//        {
//        }

//        /// <summary>
//        /// Gets the current Catalog Name from the database connection.
//        /// </summary>
//        public override string CatalogName
//        {
//            get { return _dbConnection.Database; }
//        }

//        /// <summary>
//        /// Gets the current Schema Name from the database connection.
//        /// </summary>
//        public override string SchemaName
//        {
//            get { return ""; }
//        }

//        /// <summary>
//        /// This method is the specific for the database implementation for fetching the 
//        /// DataTable representing the databases for the connection with a specified database name.
//        /// For Access this method returns null.
//        // </summary>
//        /// <param name="databaseName">The name of the database to filter by. Ignored for Access</param>
//        /// <returns>A DataTable populated with the Databases information. For Access this returns null</returns>
//        public override DataTable GetDatabasesDataTable(string databaseName)
//        {
//            return null;
//        }

//        /// <summary>
//        /// This method is the specific for the database implementation for fetching the 
//        /// DataTable representing the databases for the connection with a specified database name.
//        /// For Access this method returns null.
//        // </summary>
//        /// <param name="catalogName">The name of the database to filter by. Ignored for Access</param>
//        /// <param name="schemaName">The schema to filter by.</param>
//        /// <returns>A DataTable populated with the Databases information. For Access this returns null</returns>
//        public override DataTable GetDatabasesDataTable(string catalogName, string schemaName)
//        {
//            return GetDatabasesDataTable(catalogName);
//        }

//        //General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION,COLUMN_DEFAULT,IS_NULLABLE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION,NUMERIC_SCALE, IS_AUTO_INCREMENTING
//        //SQLSVR:   TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION,COLUMN_DEFAULT,IS_NULLABLE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,CHARACTER_OCTET_LENGTH,NUMERIC_PRECISION,NUMERIC_PRECISION_RADIX,NUMERIC_SCALE,DATETIME_PRECISION,CHARACTER_SET_CATALOG,CHARACTER_SET_SCHEMA,CHARACTER_SET_NAME,COLLATION_CATALOG
//        protected override DataTable GetColumnsDataTable(string catalogName, string schemaName, string tableName)
//        {
//            string sql = @"
//                Select *, COLUMNPROPERTY(OBJECT_ID(TABLE_NAME), COLUMN_NAME, 'IsIdentity') as IS_IDENTITY 
//                From INFORMATION_SCHEMA.COLUMNS";
//            if (!string.IsNullOrEmpty(catalogName)) SqlUtilities.AddSqlWhereParameter(ref sql, "TABLE_CATALOG", catalogName);
//            if (!string.IsNullOrEmpty(schemaName)) SqlUtilities.AddSqlWhereParameter(ref sql, "TABLE_SCHEMA", schemaName);
//            if (!string.IsNullOrEmpty(tableName)) SqlUtilities.AddSqlWhereParameter(ref sql, "TABLE_NAME", tableName);
//            sql += " ORDER BY TABLE_CATALOG, TABLE_SCHEMA, TABLE_NAME, ORDINAL_POSITION ";
//            DataTable dataTable = GetDataTable(sql);
//            DataTableUtilities.AddColumn(dataTable, DBSchemaTemplate.Naming.ColumnIsAutoIncrement, "IS_IDENTITY = 1");
//            return dataTable;
//        }

//        protected override DataTable GetDataTable(string sql)
//        {
//            OpenConnection();
//            DbDataAdapter adapter = new SqlDataAdapter(sql, (SqlConnection)_dbConnection);
//            DataTable dataTable = new DataTable();
//            adapter.Fill(dataTable);
//            return dataTable;
//        }
//    }
}