using System.Data;

namespace Firestarter.DB.Schema.Logic
{
    public class DBSchemaTemplate
    {
        public static class DataTableNaming
        {
            public const string Databases = "Databases";
            public const string DataTypes = "DataTypes";
            public const string Tables = "Tables";
            public const string Columns = "Columns";
            public const string UniqueKeys = "UniqueKeys";
            public const string PrimaryKeys = "PrimaryKeys";
            public const string KeyColumns = "KeyColumns";
            public const string ForeignKeys = "ForeignKeys";
            public const string ForeignKeyColumns = "ForeignKeyColumns";
        }

        public static class Naming
        {
            public const string CatalogName = "CATALOG_NAME";
            //public const string ColumnType = "DATA_TYPE";
            public const string SchemaName = "SCHEMA_NAME";

            public const string TableCatalog = "TABLE_CATALOG";
            public const string TableSchema = "TABLE_SCHEMA";
            public const string TableName = "TABLE_NAME";
            public const string TableType = "TABLE_TYPE";

            public const string ColumnName = "COLUMN_NAME";
            public const string ColumnDefault = "COLUMN_DEFAULT";
            public const string ColumnIsNullable = "IS_NULLABLE";
            public const string ColumnIsAutoIncrement = "IS_AUTO_INCREMENT";

            public const string KeyCatalog = "KEY_CATALOG";
            public const string KeySchema = "KEY_SCHEMA";
            public const string KeyName = "KEY_NAME";

            public const string ReferencedTableCatalog = "REFERENCED_TABLE_CATALOG";
            public const string ReferencedTableSchema = "REFERENCED_TABLE_SCHEMA";
            public const string ReferencedTableName = "REFERENCED_TABLE_NAME";
            public const string ReferencedColumnName = "REFERENCED_COLUMN_NAME";

            public const string OrdinalPosition = "ORDINAL_POSITION";

            public const string DataType = "DATA_TYPE";
            public const string CharacterMaximumLength = "CHARACTER_MAXIMUM_LENGTH";
            public const string NumericPrecision = "NUMERIC_PRECISION";
            public const string NumericScale = "NUMERIC_SCALE";

            public const string TypeName = "TypeName";
            public const string ProviderDbType = "ProviderDbType";
            public const string ColumnSize = "ColumnSize";
            public const string CreateFormat = "CreateFormat";
            public const string CreateParameters = "CreateParameters";
            public const string SystemType = "DataType";
            public const string IsAutoIncrementable = "IsAutoIncrementable";
            public const string IsBestMatch = "IsBestMatch";
            public const string IsCaseSensitive = "IsCaseSensitive";
            public const string IsFixedLength = "IsFixedLength";
            public const string IsFixedPrecisionScale = "IsFixedPrecisionScale";
            public const string IsLong = "IsLong";
            public const string IsNullable = "IsNullable";
            public const string IsSearchable = "IsSearchable";
            public const string IsSearchableWithLike = "IsSearchableWithLike";
            public const string IsUnsigned = "IsUnsigned";
            public const string MaximumScale = "MaximumScale";
            public const string MinimumScale = "MinimumScale";
            public const string IsConcurrencyType = "IsConcurrencyType";
            public const string IsLiteralSupported = "IsLiteralSupported";
            public const string LiteralPrefix = "LiteralPrefix";
            public const string LiteralSuffix = "LiteralSuffix";
        }

        public DataTable CreateDatabasesDataTable()
        {
            DataTable dataTable = new DataTable(DataTableNaming.Databases);
            dataTable.Columns.Add(Naming.CatalogName);
            dataTable.Columns.Add(Naming.SchemaName);
            return dataTable;
        }

        public DataTable CreateDataTypesDataTable()
        {
            DataTable dataTable = new DataTable(DataTableNaming.DataTypes);
            dataTable.Columns.Add(Naming.TypeName);
            dataTable.Columns.Add(Naming.ProviderDbType);
            dataTable.Columns.Add(Naming.ColumnSize);
            dataTable.Columns.Add(Naming.CreateFormat);
            dataTable.Columns.Add(Naming.CreateParameters);
            dataTable.Columns.Add(Naming.SystemType);
            dataTable.Columns.Add(Naming.IsAutoIncrementable);
            dataTable.Columns.Add(Naming.IsBestMatch);
            dataTable.Columns.Add(Naming.IsCaseSensitive);
            dataTable.Columns.Add(Naming.IsFixedLength);
            dataTable.Columns.Add(Naming.IsFixedPrecisionScale);
            dataTable.Columns.Add(Naming.IsLong);
            dataTable.Columns.Add(Naming.IsNullable);
            dataTable.Columns.Add(Naming.IsSearchable);
            dataTable.Columns.Add(Naming.IsSearchableWithLike);
            dataTable.Columns.Add(Naming.IsUnsigned);
            dataTable.Columns.Add(Naming.MaximumScale);
            dataTable.Columns.Add(Naming.MinimumScale);
            dataTable.Columns.Add(Naming.IsConcurrencyType);
            dataTable.Columns.Add(Naming.IsLiteralSupported);
            dataTable.Columns.Add(Naming.LiteralPrefix);
            dataTable.Columns.Add(Naming.LiteralSuffix);

            return dataTable;
        }

        public DataTable CreateTablesDataTable()
        {
            DataTable dataTable = new DataTable(DataTableNaming.Tables);
            dataTable.Columns.Add(Naming.TableCatalog);
            dataTable.Columns.Add(Naming.TableSchema);
            dataTable.Columns.Add(Naming.TableName);
            dataTable.Columns.Add(Naming.TableType);
            return dataTable;
        }

        public DataTable CreateColumnsDataTable()
        {
            DataTable dataTable = new DataTable(DataTableNaming.Columns);
            dataTable.Columns.Add(Naming.TableCatalog);
            dataTable.Columns.Add(Naming.TableSchema);
            dataTable.Columns.Add(Naming.TableName);
            dataTable.Columns.Add(Naming.ColumnName);
            dataTable.Columns.Add(Naming.OrdinalPosition);
            dataTable.Columns.Add(Naming.ColumnDefault);
            dataTable.Columns.Add(Naming.ColumnIsNullable);
            dataTable.Columns.Add(Naming.DataType);
            dataTable.Columns.Add(Naming.CharacterMaximumLength);
            dataTable.Columns.Add(Naming.NumericPrecision);
            dataTable.Columns.Add(Naming.NumericScale);
            dataTable.Columns.Add(Naming.ColumnIsAutoIncrement);
            return dataTable;
        }

        public DataTable CreateUniqueKeyColumnsDataTable()
        {
            DataTable dataTable = new DataTable(DataTableNaming.UniqueKeys);
            dataTable.Columns.Add(Naming.TableCatalog);
            dataTable.Columns.Add(Naming.TableSchema);
            dataTable.Columns.Add(Naming.TableName);
            dataTable.Columns.Add(Naming.KeyCatalog);
            dataTable.Columns.Add(Naming.KeySchema);
            dataTable.Columns.Add(Naming.KeyName);
            dataTable.Columns.Add(Naming.ColumnName);
            dataTable.Columns.Add(Naming.OrdinalPosition);
            return dataTable;
        }

        public DataTable CreatePrimaryKeyColumnsDataTable()
        {
            DataTable dataTable = new DataTable(DataTableNaming.PrimaryKeys);
            dataTable.Columns.Add(Naming.TableCatalog);
            dataTable.Columns.Add(Naming.TableSchema);
            dataTable.Columns.Add(Naming.TableName);
            dataTable.Columns.Add(Naming.KeyCatalog);
            dataTable.Columns.Add(Naming.KeySchema);
            dataTable.Columns.Add(Naming.KeyName);
            dataTable.Columns.Add(Naming.ColumnName);
            dataTable.Columns.Add(Naming.OrdinalPosition);
            return dataTable;
        }

        public DataTable CreateForeignKeyColumnsDataTable()
        {
            DataTable dataTable = new DataTable(DataTableNaming.ForeignKeyColumns);
            dataTable.Columns.Add(Naming.TableCatalog);
            dataTable.Columns.Add(Naming.TableSchema);
            dataTable.Columns.Add(Naming.TableName);
            dataTable.Columns.Add(Naming.KeyCatalog);
            dataTable.Columns.Add(Naming.KeySchema);
            dataTable.Columns.Add(Naming.KeyName);
            dataTable.Columns.Add(Naming.ColumnName);
            dataTable.Columns.Add(Naming.ReferencedTableCatalog);
            dataTable.Columns.Add(Naming.ReferencedTableSchema);
            dataTable.Columns.Add(Naming.ReferencedTableName);
            dataTable.Columns.Add(Naming.ReferencedColumnName);
            dataTable.Columns.Add(Naming.OrdinalPosition);
            return dataTable;
        }
    }
}
