using Firestarter.DB.Schema.BO;
using FireStarterModeller.Test;
using NUnit.Framework;

namespace Firestarter.DB.Schema.Test.BO
{
    /// <summary>
    /// Provides a place to write custom tests for DBKeyColumn objects.
    /// This file is only written once and can be changed.  The Def file
    /// attached to this as a dependent is rewritten with each regeneration
    /// and contains the standard tests for DBKeyColumn.
    /// Regenerate this test project whenever there have been changes to the
    /// business objects.
    /// If tests are failing due to a unique setup in your application,
    /// you can either override the Create methods in TestUtils, or you
    /// can add the test to the ignore list below and reimplement it here.
    /// </summary>
    public partial class TestDBKeyColumn
    {
        [TestFixtureSetUp]
        public void SetupTestFixture()
        {
            //------------------------------------------------------------
            // Use this list to ignore generated tests that are failing
            // due to a unique condition in your application.
            // Remember to reimplement the test here.
            //------------------------------------------------------------
            //_ignoreList.Add("TestMethodName", "Reason for ignoring it");
            base.SetupFixture();
        }

        [Test]
        public void Test_ToString()
        {
            //---------------Set up test pack-------------------
            DBKeyColumn dbKeyColumn = TestUtilsDBKeyColumn.CreateUnsavedValidDBKeyColumn();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(dbKeyColumn.Column);
            Assert.IsNotNull(dbKeyColumn.Key);
            //---------------Execute Test ----------------------
            string actualToString = dbKeyColumn.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual(dbKeyColumn.Key.KeyName + " - " + dbKeyColumn.Column.ColumnName, actualToString);
        }
        [Test]
        public void Test_ToString_KeyIsNull()
        {
            //---------------Set up test pack-------------------
            DBKeyColumn dbKeyColumn = TestUtilsDBKeyColumn.CreateUnsavedValidDBKeyColumn();
            dbKeyColumn.Key = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(dbKeyColumn.Key);
            Assert.IsNotNull(dbKeyColumn.Column);
            //---------------Execute Test ----------------------
            string actualToString = dbKeyColumn.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual(" - " + dbKeyColumn.Column.ColumnName, actualToString);
        } 
        [Test]
        public void Test_ToString_ColumnIsNull()
        {
            //---------------Set up test pack-------------------
            DBKeyColumn dbKeyColumn = TestUtilsDBKeyColumn.CreateUnsavedValidDBKeyColumn();
            dbKeyColumn.Column = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(dbKeyColumn.Column);
            Assert.IsNotNull(dbKeyColumn.Key);
            //---------------Execute Test ----------------------
            string actualToString = dbKeyColumn.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual(dbKeyColumn.Key + " - ", actualToString);
        }
        [Test]
        public void Test_ToString_KeyNameIsNull_KeyIdIsNull()
        {
            //---------------Set up test pack-------------------
            DBKeyColumn dbKeyColumn = TestUtilsDBKeyColumn.CreateUnsavedValidDBKeyColumn();
            dbKeyColumn.Column = null;
            dbKeyColumn.KeyID = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(dbKeyColumn.Key);
            Assert.IsNull(dbKeyColumn.Column);
            //---------------Execute Test ----------------------
            string actualToString = dbKeyColumn.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual("-unnamed keycolumn-", actualToString);
        }

        [Test]
        public void Test_ColumnName_WhenColumnNull_ShouldRetEmptyString()
        {
            //---------------Set up test pack-------------------
            DBKeyColumn pkColumn = new DBKeyColumn();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(pkColumn);
            Assert.IsNull(pkColumn.Column);
            //---------------Execute Test ----------------------
            string columnName = pkColumn.ColumnName;
            //---------------Test Result -----------------------
            Assert.IsEmpty(columnName);
        }

        [Test]
        public void Test_ColumnName_WhenColumnNotNull_ShouldRetDBColumnName()
        {
            //---------------Set up test pack-------------------
            DBKeyColumn pkColumn = new DBKeyColumn();
            string expectedColumnName = GetRandomString();
            pkColumn.Column = new DBColumn(expectedColumnName);
            //---------------Assert Precondition----------------
            Assert.IsNotNull(pkColumn);
            Assert.IsNotNull(pkColumn.Column);
            Assert.IsNotNullOrEmpty(pkColumn.Column.ColumnName);
            //---------------Execute Test ----------------------
            string columnName = pkColumn.ColumnName;
            //---------------Test Result -----------------------
            Assert.AreEqual(expectedColumnName, columnName);
        }

        private static string GetRandomString()
        {
            return TestUtilsShared.GetRandomString();
        }
    }
}