using Firestarter.DB.Schema.Logic;
using NUnit.Framework;

namespace Firestarter.DB.Schema.Test.Logic
{
    [TestFixture]
    public class TestCreateTableSqlBuilder_Access : TestCreateTableSqlBuilder
    {
        protected override CreateTableSqlBuilder CreateBuilder(string tableName)
        {
            return new CreateTableSqlBuilder_Access(tableName);
        }

        protected override string Enclose(string name)
        {
            return string.Format("[{0}]", name);
        }

        [Test]
        public override void Test_AddColumn_NotNullable_AutoIncrement()
        {
            //---------------Set up test pack-------------------
            string tableName = TestUtils.GetRandomString();
            CreateTableSqlBuilder sqlBuilderCreateTable = CreateBuilder(tableName);
            const string dataType = "int";
            string columnName = TestUtils.GetRandomString();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            sqlBuilderCreateTable.AddColumn(columnName, dataType, null, null, false, true);
            //---------------Test Result -----------------------
            string expectdSql = string.Format("{0} {1} NOT NULL IDENTITY(1,1)",
                                              Enclose(columnName), dataType);
            Assert.AreEqual(expectdSql, sqlBuilderCreateTable.Columns[0]);
        }

        [Test]
        public override void Test_AddColumn_Nullable_AutoIncrement()
        {
            //---------------Set up test pack-------------------
            string tableName = TestUtils.GetRandomString();
            CreateTableSqlBuilder sqlBuilderCreateTable = CreateBuilder(tableName);
            const string dataType = "int";
            string columnName = TestUtils.GetRandomString();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            sqlBuilderCreateTable.AddColumn(columnName, dataType, null, null, true, true);
            //---------------Test Result -----------------------
            string expectdSql = string.Format("{0} {1} NULL IDENTITY(1,1)",
                                              Enclose(columnName), dataType);
            Assert.AreEqual(expectdSql, sqlBuilderCreateTable.Columns[0]);
        }

        [Test]
        public override void Test_AddForeignKey()
        {
            //---------------Set up test pack-------------------
            string tableName = TestUtils.GetRandomString();
            CreateTableSqlBuilder sqlBuilderCreateTable = CreateBuilder(tableName);
            string referencedTableName = TestUtils.GetRandomString();
            string foreignKeyName = TestUtils.GetRandomString();
            const string parentColumnNameList = "field1FK, field2FK";
            string expectedParentColumnNameList = Enclose("field1FK") + ", " + Enclose("field2FK");
            const string referencedColumnNameList = "field1, field2";
            string expectedReferencedColumnNameList = Enclose("field1") + ", " + Enclose("field2");
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            sqlBuilderCreateTable.AddForeignKey
                (foreignKeyName, parentColumnNameList, referencedTableName, referencedColumnNameList);
            //---------------Test Result -----------------------
            Assert.AreEqual(1, sqlBuilderCreateTable.ForeignKeys.Count);
            string expectdFKString = string.Format
                ("CONSTRAINT {0} FOREIGN KEY ({1}) REFERENCES {2} ({3})",
                Enclose(foreignKeyName), expectedParentColumnNameList,
                Enclose(referencedTableName), expectedReferencedColumnNameList);
            Assert.AreEqual(expectdFKString, sqlBuilderCreateTable.ForeignKeys[0]);
        }

        [Test]
        public override void Test_ForeignKeyGetSQL()
        {
            //---------------Set up test pack-------------------
            string tableName = TestUtils.GetRandomString();
            CreateTableSqlBuilder_SqlServer sqlBuilder_SqlBuilderCreateTable = new CreateTableSqlBuilder_SqlServer(tableName);
            string referencedTableName = TestUtils.GetRandomString();
            string foreignKeyName = TestUtils.GetRandomString();
            const string parentColumnNameList = "field1FK, field2FK";
            string expectedParentColumnNameList = Enclose("field1FK") + ", " + Enclose("field2FK");
            const string referencedColumnNameList = "field1, field2";
            string expectedReferencedColumnNameList = Enclose("field1") + ", " + Enclose("field2");
            sqlBuilder_SqlBuilderCreateTable.AddForeignKey
                (foreignKeyName, parentColumnNameList, referencedTableName, referencedColumnNameList);
            //---------------Assert Precondition----------------
            Assert.AreEqual(1, sqlBuilder_SqlBuilderCreateTable.ForeignKeys.Count);
            //---------------Execute Test ----------------------
            string sql = sqlBuilder_SqlBuilderCreateTable.GetSql();
            //---------------Test Result -----------------------
            string expectdFKString = string.Format
                ("CONSTRAINT {0} FOREIGN KEY ({1}) REFERENCES {2} ({3})",
                Enclose(foreignKeyName), expectedParentColumnNameList,
                Enclose(referencedTableName), expectedReferencedColumnNameList);
            StringAssert.Contains(expectdFKString, sql);
        }
    }
}