using System;
using System.Collections.Generic;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using Habanero.Base.Exceptions;

namespace Firestarter.DB_DomainModel.Logic
{
    public class DBToDMObjectIdentityImporter : IDBToDMObjectIdentityImporter
    {
        public DMObjectIdentity Import(DMClass dmClass, DBPrimaryKey dbPrimaryKey, out IList<string> errorMessages)
        {
            errorMessages = new List<string>(4);
            if (dmClass == null) throw new ArgumentNullException("dmClass");
            if (dbPrimaryKey == null) throw new ArgumentNullException("dbPrimaryKey");
//            if (dmClass == null)
//                throw new HabaneroArgumentException
//                    ("dmClass", "You cannot import the primary key since the business object is null");

            DMObjectIdentity objectIdentity = GetMappedObjectIdentity(dmClass, dbPrimaryKey);

            foreach (DMProperty property in objectIdentity.Properties.Clone())
            {
                DMProperty dmProperty = property;
                if (!dbPrimaryKey.PrimaryKeyColumns.Exists(column => 
                    column.Column.ColumnName == dmProperty.MappedDBColumnName))
                {
                    objectIdentity.Properties.Remove(dmProperty);
                }
            }
            foreach (DBColumn columnInfo in dbPrimaryKey.Columns)
            {
                DMProperty property = dmClass.FindBOPropertyByColumnId(columnInfo.ColumnID);
                if (property == null)
                {
                    string expectedMessage = string.Format("The primary key for business object '{0}' could not be "
                                                           + "created since the column '{1}' do not exist on the business object"
                                                           , dmClass.ClassNameBO, columnInfo.ColumnName);
                    errorMessages.Add(expectedMessage);
                    return null;
                }
                objectIdentity.Properties.Add(property);
            }
            return objectIdentity;
        }

        private DMObjectIdentity GetMappedObjectIdentity(DMClass dmClass, DBPrimaryKey dbPrimaryKey)
        {
            DMObjectIdentity objectIdentity = dbPrimaryKey.GetMappedDMObjectIdentity();
            if (objectIdentity != null && objectIdentity.Class != dmClass)
            {
                objectIdentity.MappedDBPrimaryKey = null;// objectIdentity.Class.MappedTable.PrimaryKeyDB;
                objectIdentity = null;
            }
            if (objectIdentity == null)
            {
                objectIdentity = dmClass.GetObjectID() ?? new DMObjectIdentity { Class = dmClass };
            }
            objectIdentity.MappedDBPrimaryKey = dbPrimaryKey;
            return objectIdentity;
        }
    }
}