using System;
using Firestarter.DB.Schema.BO;
using Firestarter.DB.Schema.Logic;
using FireStarterModeller.BO;

namespace Firestarter.DB_DomainModel.Logic
{
    public class DBToDMPropertyImporter : IDBToDMPropertyImporter
    {
        public IDBToDMPropertyUpdater DBToDMPropertyUpdater { get; private set; }

        public DBToDMPropertyImporter(IDBToDMPropertyUpdater dbToDMPropertyUpdater)
        {
            if (dbToDMPropertyUpdater == null) throw new ArgumentNullException("dbToDMPropertyUpdater");
            DBToDMPropertyUpdater = dbToDMPropertyUpdater;
        }

        public DMProperty Import(DMClass dmClass, DBColumn dbColumn)
        {
            if (dmClass == null) throw new ArgumentNullException("dmClass");
            if (dbColumn == null) throw new ArgumentNullException("dbColumn");

            // Note: Shouldn't this next line just use the relationship??? This may be because it could be a different class?!?
            DMProperty foundDMProperty = dmClass.FindBOPropertyByColumnId(dbColumn.ColumnID);
            if (foundDMProperty != null)
            {
                foundDMProperty.MappedDBColumnName = dbColumn.ColumnName;
                return UpdateProperty(foundDMProperty);
            }
            foundDMProperty = dmClass.FindBOPropertyByColumnName(dbColumn.ColumnName);
            if (foundDMProperty != null)
            {
                foundDMProperty.MappedDBColumnID = dbColumn.ColumnID;
                return UpdateProperty(foundDMProperty);
            }

            DMProperty newProperty = new DMProperty
                                         {
                                             MappedDBColumn = dbColumn,
                                             MappedDBColumnName = dbColumn.ColumnName
                                         };
            UpdateProperty(newProperty);

            
            //            prop.DefaultValue = dbColumn.DefaultValue;
            dmClass.Properties.Add(newProperty);
            return newProperty;
            //            return prop;
        }

        private DMProperty UpdateProperty(DMProperty property)
        {
            DBColumn mappedDBColumn = property.MappedDBColumn;
            DBToDMPropertyUpdater.UpdateValues(mappedDBColumn, property);
            return property;
        }
    }
}