using System;
using FireStarterModeller.BO;
using Habanero.BO;
using NUnit.Framework;

namespace FireStarterModeller.Test.BO
{
    /// <summary>
    /// Provides a place to write custom tests for DMInheritanceRelationship objects.
    /// This file is only written once and can be changed.  The Def file
    /// attached to this as a dependent is rewritten with each regeneration
    /// and contains the standard tests for DMInheritanceRelationship.
    /// Regenerate this test project whenever there have been changes to the
    /// business objects.
    /// If tests are failing due to a unique setup in your application,
    /// you can either override the Create methods in TestUtils, or you
    /// can add the test to the ignore list below and reimplement it here.
    /// </summary>
    public partial class TestDMInheritanceRelationship
    {
        [SetUp]
        public override void SetupTest()
        {
            //Runs every time that any testmethod is executed
            base.SetupTest();
        }

        [TestFixtureSetUp]
        public void TestFixtureSetup()
        {
            //------------------------------------------------------------
            // Use this list to ignore generated tests that are failing
            // due to a unique condition in your application.
            // Remember to reimplement the test here.
            //------------------------------------------------------------
            //_ignoreList.Add("TestMethodName", "Reason for ignoring it");
            base.SetupFixture();
        }

        [TearDown]
        public override void TearDownTest()
        {
            //runs every time any testmethod is complete
            base.TearDownTest();
        }
//        [Test]
//        public void Test_ToString()
//        {
//            //---------------Set up test pack-------------------
//            DMInheritanceRelationship dmInheritanceRelationship = TestUtilsDMInheritanceRelationship.CreateUnsavedValidDMInheritanceRelationship();
//            //---------------Assert Precondition----------------
//            Assert.IsNotNull(dmInheritanceRelationship.Class, "SuperClass.Class Should not be null");
//            //---------------Execute Test ----------------------
//            string actualToString = dmInheritanceRelationship.ToString();
//            //---------------Test Result -----------------------
//            Assert.AreEqual(dmInheritanceRelationship.Class, actualToString);
//        }
        [Test]
        public void Test_ToString_SuperClassNameNull()
        {
            //---------------Set up test pack-------------------
            DMInheritanceRelationship dmInheritanceRelationship = TestUtilsDMInheritanceRelationship.CreateUnsavedValidDMInheritanceRelationship();
            dmInheritanceRelationship.SubClass = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(dmInheritanceRelationship.SubClass, "SuperClassName Should be null");
            //---------------Execute Test ----------------------
            string actualToString = dmInheritanceRelationship.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual(dmInheritanceRelationship.InheritanceRelationshipID.ToString(), actualToString);
        }
//        [Test]
//        public void Test_ToString_SuperClassNameNull_SuperClassIDNull()
//        {
//            //---------------Set up test pack-------------------
//            DMInheritanceRelationship dmInheritanceRelationship = TestUtilsDMInheritanceRelationship.CreateUnsavedValidDMInheritanceRelationship();
//            dmInheritanceRelationship.SuperClassName = null;
//            dmInheritanceRelationship.SuperClassID = null;
//            //---------------Assert Precondition----------------
//            Assert.IsTrue(String.IsNullOrEmpty(dmInheritanceRelationship.SuperClassName), "SuperClassName Should be null");
//            Assert.IsNull(dmInheritanceRelationship.SuperClassID);
//            //---------------Execute Test ----------------------
//            string actualToString = dmInheritanceRelationship.ToString();
//            //---------------Test Result -----------------------
//            Assert.AreEqual("-unnamed solution-", actualToString);
//        }
        [Test]
        public void Test_InsertNewSuperClassInfo()
        {
            //---------------Set up test pack-------------------
            DMInheritanceRelationship dmInheritanceRelationship = TestUtilsShared.CreateUnsavedValidSuperClassInfo();

            //---------------Assert Precondition----------------
            Assert.IsTrue(dmInheritanceRelationship.Status.IsNew);

            //---------------Execute Test ----------------------
            dmInheritanceRelationship.Save();

            //---------------Test Result -----------------------
            Assert.IsFalse(dmInheritanceRelationship.Status.IsNew);
        }

        [Test]
        public void Test_InsertNewSuperClassInfo_Invalid()
        {
            //---------------Set up test pack-------------------
            DMInheritanceRelationship dmInheritanceRelationship = new DMInheritanceRelationship();

            //---------------Assert Precondition----------------
            Assert.IsTrue(dmInheritanceRelationship.Status.IsNew);

            //---------------Execute Test ----------------------
            try
            {
                dmInheritanceRelationship.Save();
                Assert.Fail("expected Err");
            }
            //---------------Test Result -----------------------
            catch (BusObjectInAnInvalidStateException ex)
            {
                StringAssert.Contains("Super Class' is a compulsory field and has no value.", ex.Message);
            }
            Assert.IsTrue(dmInheritanceRelationship.Status.IsNew);
        }

        [Test]
        public void Test_DeleteSuperClassInfo()
        {
            //---------------Set up test pack-------------------
            DMInheritanceRelationship dmInheritanceRelationship = TestUtilsShared.CreateSavedSuperClassInfo();

            //---------------Assert Precondition----------------
            Assert.IsFalse(dmInheritanceRelationship.Status.IsNew);
            string message;
            Assert.IsTrue(dmInheritanceRelationship.IsDeletable(out message));

            //---------------Execute Test ----------------------
            dmInheritanceRelationship.MarkForDelete();
            dmInheritanceRelationship.Save();

            //---------------Test Result -----------------------  
            Assert.IsTrue(dmInheritanceRelationship.Status.IsNew);
            Assert.IsTrue(dmInheritanceRelationship.Status.IsDeleted);
        }

        [Test]
        public void Test_UpdateSuperClassInfo()
        {
            //---------------Set up test pack-------------------
            DMInheritanceRelationship dmInheritanceRelationship = TestUtilsShared.CreateSavedSuperClassInfo();

            //---------------Assert Precondition----------------
            Assert.IsFalse(dmInheritanceRelationship.Status.IsNew);
            string message;
            Assert.IsTrue(dmInheritanceRelationship.IsEditable(out message));

            //---------------Execute Test ----------------------
            dmInheritanceRelationship.Generated = true;
            dmInheritanceRelationship.Save();

            //---------------Test Result -----------------------
            Assert.IsFalse(dmInheritanceRelationship.Status.IsNew);
        }

        [Test]
        public void Test_SingleTableInheritance_DiscriminatorCompulsory()
        {
            //---------------Set up test pack-------------------
            DMInheritanceRelationship dmInheritanceRelationship = TestUtilsShared.CreateUnsavedValidSuperClassInfo();
            dmInheritanceRelationship.InheritanceType = Habanero.BO.ClassDefinition.ORMapping.SingleTableInheritance;
            //---------------Execute Test ----------------------
            try
            {
                dmInheritanceRelationship.Discriminator = null;
                dmInheritanceRelationship.Save();
                Assert.Fail("expected Err");
            }
            //---------------Test Result -----------------------
            catch (Exception ex)
            {
                StringAssert.Contains("The Discriminator is compulsory for Single Table Inheritance", ex.Message);
            }
            //---------------Tear Down -------------------------

        }

        [Test]
        public void Test_SingleTableInheritance()
        {
            //---------------Set up test pack-------------------
            DMInheritanceRelationship dmInheritanceRelationship = new DMInheritanceRelationship();
            dmInheritanceRelationship.InheritanceType = Habanero.BO.ClassDefinition.ORMapping.SingleTableInheritance;
            TestUtilsShared.UpdateSuperClassInfoWithValidData(dmInheritanceRelationship);
            //------------Assert Preconditions--------------------
            Assert.AreEqual(Habanero.BO.ClassDefinition.ORMapping.SingleTableInheritance, dmInheritanceRelationship.InheritanceType);
            Assert.IsNotNull(dmInheritanceRelationship.Discriminator);
            //---------------Execute Test ----------------------

             dmInheritanceRelationship.Save();
            //---------------Test Result -----------------------
            Assert.IsFalse(dmInheritanceRelationship.Status.IsDirty);
            //---------------Tear Down -------------------------

        }

    }
}