using System;
using FireStarter.Base;
using FireStarterModeller.BO;
using Habanero.Base.Exceptions;
using NUnit.Framework;

namespace FireStarterModeller.Test.BO
{
    /// <summary>
    /// Provides a place to write custom tests for UIControlMapperType objects.
    /// This file is only written once and can be changed.  The Def file
    /// attached to this as a dependent is rewritten with each regeneration
    /// and contains the standard tests for UIControlMapperType.
    /// Regenerate this test project whenever there have been changes to the
    /// business objects.
    /// If tests are failing due to a unique setup in your application,
    /// you can either override the Create methods in TestUtils, or you
    /// can add the test to the ignore list below and reimplement it here.
    /// </summary>
    public partial class TestUIControlMapperType
    {
        [TestFixtureSetUp]
        public void SetupTestFixture()
        {
            //------------------------------------------------------------
            // Use this list to ignore generated tests that are failing
            // due to a unique condition in your application.
            // Remember to reimplement the test here.
            //------------------------------------------------------------
            //_ignoreList.Add("TestMethodName", "Reason for ignoring it");
            base.SetupFixture();
        }

        [Test]
        public void Test_ToString()
        {
            //---------------Set up test pack-------------------
            UIControlMapperType uiControlMapperType =
                TestUtilsUIControlMapperType.CreateUnsavedValidUIControlMapperType();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(uiControlMapperType.Solution);
            TestUtilsShared.AssertStringNotEmpty(uiControlMapperType.AssemblyName, "uiControlMapperType.AssemblyName");
            TestUtilsShared.AssertStringNotEmpty(uiControlMapperType.TypeName, "uiControlMapperType.TypeName");
            //---------------Execute Test ----------------------
            string actualToString = uiControlMapperType.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual
                (uiControlMapperType.Solution + " - " + uiControlMapperType.AssemblyName + "."
                 + uiControlMapperType.TypeName, actualToString);
        }

        [Test]
        public void Test_ToString_SolutionNull()
        {
            //---------------Set up test pack-------------------
            UIControlMapperType uiControlMapperType =
                TestUtilsUIControlMapperType.CreateUnsavedValidUIControlMapperType();
            uiControlMapperType.Solution = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(uiControlMapperType.Solution);
            TestUtilsShared.AssertStringNotEmpty(uiControlMapperType.AssemblyName, "uiControlMapperType.AssemblyName");
            //---------------Execute Test ----------------------
            string actualToString = uiControlMapperType.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual
                (" - " + uiControlMapperType.AssemblyName + "." + uiControlMapperType.TypeName, actualToString);
        }

        [Test]
        public void Test_ToString_SolutionNull_AssemblyNameNull()
        {
            //---------------Set up test pack-------------------
            UIControlMapperType uiControlMapperType =
                TestUtilsUIControlMapperType.CreateUnsavedValidUIControlMapperType();
            uiControlMapperType.AssemblyName = null;
            uiControlMapperType.Solution = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(uiControlMapperType.Solution);
            TestUtilsShared.AssertStringEmpty(uiControlMapperType.AssemblyName, "uiControlMapperType.AssemblyName");
            //---------------Execute Test ----------------------
            string actualToString = uiControlMapperType.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual(" - ." + uiControlMapperType.TypeName, actualToString);
        }

        [Test]
        public void Test_ToString_SolutionNull_AssemblyNameNull_TypeNameNull()
        {
            //---------------Set up test pack-------------------
            UIControlMapperType uiControlMapperType =
                TestUtilsUIControlMapperType.CreateUnsavedValidUIControlMapperType();
            uiControlMapperType.AssemblyName = null;
            uiControlMapperType.Solution = null;
            uiControlMapperType.TypeName = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(uiControlMapperType.Solution);
            TestUtilsShared.AssertStringEmpty(uiControlMapperType.AssemblyName, "uiControlMapperType.AssemblyName");
            TestUtilsShared.AssertStringEmpty(uiControlMapperType.TypeName, "uiControlMapperType.TypeName");
            //---------------Execute Test ----------------------
            string actualToString = uiControlMapperType.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual(uiControlMapperType.UIControlMapperTypeID.ToString(), actualToString);
        }

        [Test]
        public void Test_ToString_SolutionNull_AssemblyNameNull_UIControlMapperTypeIDNull()
        {
            //---------------Set up test pack-------------------
            UIControlMapperType uiControlMapperType =
                TestUtilsUIControlMapperType.CreateUnsavedValidUIControlMapperType();
            uiControlMapperType.Solution = null;
            uiControlMapperType.AssemblyName = null;
            uiControlMapperType.TypeName = null;
            uiControlMapperType.UIControlMapperTypeID = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(uiControlMapperType.Solution);
            Assert.IsNull(uiControlMapperType.AssemblyName);
            Assert.IsNull(uiControlMapperType.UIControlMapperTypeID);
            //---------------Execute Test ----------------------
            string actualToString = uiControlMapperType.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual("-unnamed UIControlMapperType-", actualToString);
        }
        [Test]
        public void Test_GetParent()
        {
            //---------------Set up test pack-------------------
            DMSolution solution = new DMSolution();
            UIControlMapperType objects = new UIControlMapperType();
            solution.UIControlMapperTypes.Add(objects);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IModelElement parent = ((IModelElement)objects).Parent;
            //---------------Test Result -----------------------
            Assert.AreSame(solution, parent);
        }

        [Test]
        public void Test_SetParentExceptionThrown()
        {
            //---------------Set up test pack-------------------
            IModelElement prop = new UIControlMapperType();

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            Exception exceptionThrown = null;
            try
            {
                prop.Parent = new DMSolution();
            }
            catch (Exception ex)
            {
                exceptionThrown = ex;
            }
            //---------------Test Result -----------------------
            Assert.IsNotNull(exceptionThrown);
            Assert.IsInstanceOf(typeof(HabaneroDeveloperException), exceptionThrown);
            StringAssert.Contains
                ("Set the parent of the UIControlMapperType by setting the related Solution object",
                 exceptionThrown.Message);
        }
 
    }
}